#include <stdio.h> // for puts
#include <utility> // for std::move

struct T
{
    T           ()         noexcept { }
    T           (T const&) noexcept { puts("copy-ctor"); }
    T           (T&&)      noexcept { puts("move-ctor"); }
    T& operator=(T const&) noexcept { puts("copy-asgn"); return *this; }
    T& operator=(T&&)      noexcept { puts("move-asgn"); return *this; }
};

inline void fn(T& ) { puts("l"); }
inline void fn(T&&) { puts("r"); }

static void example()
{
    T   val{};
    T&  lvr = val;
    T&& rvr = T{};

    fn(val); // l
    fn(lvr); // l
    fn(rvr); // l
    fn(T{}); // r
    puts("");
    fn(std::move(val)); // r
    fn(std::move(lvr)); // r
    fn(std::move(rvr)); // r
    fn(std::move(T{})); // r
    puts("");
    fn(static_cast<decltype(val)>(val));   // r, decltype(val) === T    Copies val!
    fn(static_cast<decltype(lvr)>(lvr));   // l, decltype(lvr) === T&
    fn(static_cast<decltype(rvr)>(rvr));   // r, decltype(rvr) === T&&
    fn(static_cast<decltype(T{})>(T{}));   // r, decltype(T{}) === T
    puts("");
    fn(static_cast<decltype(val)&&>(val)); // r, decltype(val) === T    decltype(val)&& === T&&
    fn(static_cast<decltype(lvr)&&>(lvr)); // l, decltype(lvr) === T&   decltype(lvr)&& === T&
    fn(static_cast<decltype(rvr)&&>(rvr)); // r, decltype(rvr) === T&&  decltype(rvr)&& === T&&
    fn(static_cast<decltype(T{})&&>(T{})); // r, decltype(T{}) === T    decltype(T{})&& === T&&
}

// NOTE! By default MSVC will incorrectly treat the cast on line 32 as an lvalue.
// There are several options you can use for correcting this behavior:
//
// 1. /permissive-     Puts the entire compiler in a more standards compliant mode (recommended)
// 2. /Za              Disables most compiler extensions.
// 3. /Zc:rvalueCast-  Prevent this specific incorrect behavior.
//
// You have to set the 1st one under: C/C++ => Command line => Additional options
// The 2nd and 3rd are part of the section: C/C++ => Language
