#include <utility>
#include <type_traits>

#define  IS_LVALUE(EXPR) ( std::is_lvalue_reference_v<decltype((EXPR))>)
#define  IS_XVALUE(EXPR) ( std::is_rvalue_reference_v<decltype((EXPR))>)
#define IS_PRVALUE(EXPR) (!std::is_reference_v       <decltype((EXPR))>)

template <typename Fun, typename... P>
decltype(auto) passthrough(Fun fn, P&&... v)
{
    return fn(std::forward<P>(v)...);
}

int   idt_integer(int& v) { return v; }
int&  lvr_integer(int& v) { return v; }
int&& rvr_integer(int& v) { return static_cast<int&&>(v); }

static void example()
{
    int x{ 5 };

    static_assert(IS_PRVALUE( idt_integer(x) ));
    static_assert( IS_LVALUE( lvr_integer(x) ));
    static_assert( IS_XVALUE( rvr_integer(x) ));
    static_assert( IS_XVALUE( std::move<int&>(x) ));

    static_assert(IS_PRVALUE( passthrough(idt_integer, x) ));
    static_assert( IS_LVALUE( passthrough(lvr_integer, x) ));
    static_assert( IS_XVALUE( passthrough(rvr_integer, x) ));
    static_assert( IS_XVALUE( passthrough(std::move<int&>, x) ));
}
