var CodeSample;
(function (CodeSample) {
    class NoteFormatter {
        preprocess_line(line) {
            line.notes = [];
        }
        process_set(lines, set) {
            let active_notes = [];
            set.forEach((keyframe, keyframe_index) => {
                for (const action of keyframe) {
                    switch (action.type) {
                        case 'add_note':
                            {
                                const line = lines[action.line_index];
                                const line_number = $('<span class="collapse">');
                                const source_code = $('<div class="note collapse">')
                                    .addClass('note-' + action.note_type);
                                $('<div class="content">')
                                    .text(' '.repeat(action.start_column) + action.content)
                                    .appendTo(source_code);
                                line.notes.push({
                                    line_number: line_number,
                                    source_code: source_code,
                                    toggle_frames: [keyframe_index]
                                });
                                active_notes.push(line.notes[line.notes.length - 1]);
                                break;
                            }
                        case 'clear_notes':
                            {
                                active_notes.forEach(x => x.toggle_frames.push(keyframe_index));
                                active_notes = [];
                                break;
                            }
                    }
                }
            });
        }
        render(lines, set) {
            for (const line of lines) {
                for (const note of line.notes) {
                    line.line_number.push(note.line_number);
                    line.source_code.push(note.source_code);
                }
            }
            return set.map((keyframe, keyframe_index) => {
                const toggles = [];
                for (const line of lines) {
                    for (const note of line.notes) {
                        for (const toggle of note.toggle_frames) {
                            if (toggle === keyframe_index) {
                                toggles.push(note.line_number, note.source_code);
                            }
                        }
                    }
                }
                return toggles;
            });
        }
    }
    CodeSample.Code.formatters.push(new NoteFormatter());
})(CodeSample || (CodeSample = {}));
//# sourceMappingURL=notes.js.map