const BUNDLE_DATA = {
	"content/10_introduction.html": "<section data-footnote=\"Introduction\">\r\n    <article class=\"intermediate\">\r\n        <h1>Modern C++: Move semantics</h1>\r\n        <h2>A series of workshops covering modern features of C++</h2>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Series overview</h1>\r\n        <p>4 workshops:</p>\r\n        <ul>\r\n            <li>Fundamentals</li>\r\n            <li>Templates</li>\r\n            <li><em>Move semantics &amp; rvalue references</em></li>\r\n            <li>Metaprogramming</li>\r\n        </ul>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Agenda</h1>\r\n        <ol>\r\n            <li data-fade=\"0\"><strong>Lecture:</strong> Move semantics (practical)</li>\r\n            <li data-fade=\"1\"><strong>Lecture:</strong> rvalue references (theoretical)</li>\r\n            <li data-fade=\"2\"><strong>Lecture:</strong> Perfect forwarding (mixed)</li>\r\n            <li data-fade=\"3\"><strong>Exercises:</strong> 1 quiz, 2 exercises.</li>\r\n        </ol>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Before we start</h1>\r\n        <p data-fade=\"0\"><em>Get latest version of Visual Studio 2017!</em></p>\r\n        <p data-fade=\"1\">\r\n            Contact information:\r\n            <table data-fade=\"1\">\r\n                <tr>\r\n                    <th>E-mail</th>\r\n                    <td>143042@nhtv.nl</td>\r\n                </tr>\r\n                <tr>\r\n                    <th>Slack</th>\r\n                    <td>Caspar143042</td>\r\n                </tr>\r\n                <tr>\r\n                    <th>Discord</th>\r\n                    <td>Aidiakapi#2177</td>\r\n                </tr>\r\n            </table>\r\n        </p>\r\n        <p data-fade=\"2\"><em>Please give me feedback!</em> <a href=\"https://goo.gl/forms/zVXOH97o1iyB4qIj1\">https://goo.gl/forms/zVXOH97o1iyB4qIj1</a></p>\r\n    </article>\r\n</section>\r\n",
	"content/20_move_semantics.html": "<section data-footnote=\"Move Semantics\">\r\n    <article class=\"intermediate\">\r\n        <h1>Move Semantics</h1>\r\n        <h2>What is it, why should you care?</h2>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>When copying makes no sense</h1>\r\n        <p>Some objects should logically not be copied.</p>\r\n        <code data-source=\"20_non_copyable\"></code>\r\n        <p data-fade=\"12\">Copying a <code>std::unique_ptr</code> is not allowed. Why?</p>\r\n        <p data-fade=\"13\">It is a unique owner of the resource. Copying would imply at least 2 owners.</p>\r\n        <p data-fade=\"14\">A <code>std::unique_ptr</code> can be moved instead!</p>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>When it makes no sense to copy</h1>\r\n        <p>Some objects you don't want to copy.</p>\r\n        <code data-source=\"21_unnecessary_copy\"></code>\r\n        <p data-fade=\"20\">Moving here avoids an unnecessary copy.</p>\r\n        <p data-fade=\"21\">The state of bullets is unspecified but valid.<sup><a href=\"https://stackoverflow.com/a/17735913/613176\">[SO]</a></sup></p>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Motivation</h1>\r\n        <p>Reasons for moving:</p>\r\n        <ol>\r\n            <li data-fade=\"0\">\r\n                When a type logically <strong>cannot be copied</strong>.\r\n                <ul>\r\n                    <li data-fade=\"1\"><code>std::unique_ptr</code></li>\r\n                    <li data-fade=\"2\"><code>std::unique_lock</code></li>\r\n                    <li data-fade=\"3\"><code>std::basic_istream</code> &amp; <code>std::basic_ostream</code></li>\r\n                    <li data-fade=\"4\">Objects living in VRAM.</li>\r\n                    <li data-fade=\"5\">Network sockets.</li>\r\n                </ul>\r\n            </li>\r\n            <li data-fade=\"10\">\r\n                Avoid copies to improve <strong>performance</strong>.\r\n                <ul>\r\n                    <li data-fade=\"11\">Any copyable resource.</li>\r\n                    <li data-fade=\"12\">Collections.</li>\r\n                </ul>\r\n            </li>\r\n        </ol>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Copy semantics</h1>\r\n        <code data-source=\"22_surface\" data-set=\"copy_issues\"></code>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Move semantics</h1>\r\n        <code data-source=\"22_surface\" data-set=\"move_semantics\"></code>\r\n    </article>\r\n\r\n    <!--<article>\r\n        <h1>Default operations</h1>\r\n        <p>By now, you've seen all default operations, let's define them more explicitly.</p>\r\n        <code data-source=\"23_default_operations\"></code>\r\n    </article>-->\r\n\r\n    <article>\r\n        <h1>Default operations</h1>\r\n        <p>Info about default operations:</p>\r\n        <ul>\r\n            <li data-fade=\"0\">6 default operations: default constructor, copy constructor, copy assignment, move constructor, move assignment, destructor.</li>\r\n            <li data-fade=\"1\">Implicitly or explicit generated. (<code>= default</code>)</li>\r\n            <li data-fade=\"2\">Default implementation applies it to each field.</li>\r\n            <li data-fade=\"3\"><code>noexcept</code> if operation on all fields is <code>noexcept</code>.</li>\r\n        </ul>\r\n        <p data-fade=\"10\">Default implementation is <em>similar</em> but not identical to this:</p>\r\n        <code data-source=\"23_default_operations\" data-fade=\"10;18\" data-frame-offset=\"11\"></code>\r\n        <p data-fade=\"20\">The default implementation may be <em>trivial</em>.<sup><a href=\"https://stackoverflow.com/a/6496703\">[SO]</a></sup></p>\r\n        <p data-fade=\"21\"><a href=\"https://blog.tartanllama.xyz/initialization-is-bonkers/\">Initialization is bonkers.</a></p>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Summary</h1>\r\n        <p>What we covered:</p>\r\n        <ul>\r\n            <li data-fade=\"0\">Non-copyable objects might be moveable.</li>\r\n            <li data-fade=\"1\">Moving is generally faster.</li>\r\n            <li data-fade=\"2\">Objects moved from <strong>should be</strong> in a valid state.</li>\r\n            <li data-fade=\"3\">In standard library, state of moved from objects is <em>unspecified</em>.</li>\r\n            <li data-fade=\"4\">For specific types (like <code>std::unique_ptr</code>), the moved-from state is specified.</li>\r\n            <li data-fade=\"5\">You can implement your own move constructor and assignment.</li>\r\n            <li data-fade=\"6\">Default implementation is applying that same operation to each field.</li>\r\n        </ul>\r\n    </article>\r\n</section>\r\n",
	"content/30_references.html": "<section data-footnote=\"References\">\r\n    <article class=\"intermediate\">\r\n        <h1>References</h1>\r\n        <h2><code>&</code> and <code>&&</code> <span style=\"font-family: 'Fira Code';\"><==</span> these things</h2>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>So what are <code>&</code> and <code>&&</code>?</h1>\r\n        <ul>\r\n            <li data-fade=\"0\"><code>T&&</code> is an <strong>rvalue reference</strong> to T.</li>\r\n            <li data-fade=\"1\"><code>T&</code><span style=\"font-family: 'Fira Code';\">&nbsp;</span> is an <strong>lvalue reference</strong> to T.</li>\r\n            <li data-fade=\"2\">References are just an <em>alias</em> to an object.</li>\r\n            <li data-fade=\"3\">References to references don't exist.</li>\r\n        </ul>\r\n        <p data-fade=\"10\">The next question is, what are <strong>lvalues</strong> and <strong>rvalues</strong>?</p>\r\n        <p data-fade=\"11\">These are known <em>expression categories</em> or <strong>value categories</strong>.</p>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>What are expression categories?</h1>\r\n        <ul>\r\n            <li data-fade=\"0\">Category of an expression. What is an expression?</li>\r\n            <li data-fade=\"1\"><q>An expression is a sequence of operators and their operands, that specifies a computation.</q> <sup><a href=\"http://en.cppreference.com/w/cpp/language/expressions\">[C++ Ref]</a></sup></li>\r\n            <li data-fade=\"2\">\r\n                Expressions include:\r\n                <ul>\r\n                    <li data-fade=\"3\">Any operator (including call, and member access).</li>\r\n                    <li data-fade=\"4\">Casts.</li>\r\n                    <li data-fade=\"5\">Some keywords (new, delete, sizeof, ...).</li>\r\n                    <li data-fade=\"6\">Names and constants.</li>\r\n                </ul>\r\n            </li>\r\n            <li data-fade=\"7\"><em>Each</em> expression is either an <em>lvalue</em> or <em>rvalue</em>.</li>\r\n            <li data-fade=\"8\">There are 2 types of <strong>rvalues</strong>: <em>xvalue</em> and <em>prvalue</em>.</li>\r\n        </ul>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>History of the terminology</h1>\r\n        <p>Historically <em>lvalue</em> meant <strong>left value</strong>, <em>rvalue</em> meant <strong>right value</strong>.</p>\r\n        <code data-source=\"30_historic\" data-fade=\"1\" data-frame-offset=\"2\"></code>\r\n        <p data-fade=\"30\">Left and right values are not, and never were accurate.</p>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Distinguising expression categories</h1>\r\n        <p>Rules of thumb: Any expression is an <em>lvalue</em>, if ...</p>\r\n        <ul>\r\n            <li data-fade=\"0\">...you can take the address of it.</li>\r\n            <li data-fade=\"1\">...it has a name.</li>\r\n            <li data-fade=\"2\">...it has type <code>T&</code>.</li>\r\n        </ul>\r\n        <code data-source=\"22_surface\" data-set=\"expression_categories\" data-fade=\"4\" data-frame-offset=\"5\"></code>\r\n    </article>\r\n    \r\n    <article>\r\n        <h1><code>T&&</code> rvalue references</h1>\r\n        <p>A parameter or variable of type <code>T&&</code> can reference an <em>rvalue</em>.</p>\r\n        <code data-source=\"31_rvalue_references\" data-fade=\"1\" data-frame-offset=\"2\"></code>\r\n        <p data-fade=\"30\">An <em>rvalue reference</em> is still a reference!</p>\r\n        <p data-fade=\"31\"><em>References</em> alias objects.</p>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Function overloading</h1>\r\n        <code data-source=\"32_function_overloading\"></code>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Function overloading (cont)</h1>\r\n        <p>Summarizing</p>\r\n        <ul>\r\n            <li data-fade=\"0\"><strong>Reminder:</strong> A variable or parameter of <code>T&&</code> is still an <em>lvalue</em>.</li>\r\n            <li data-fade=\"1\">Overloading between <code>T&</code> and <code>T&&</code> is done based on <em>expression category</em>.</li>\r\n            <li data-fade=\"2\">\r\n                References to references collapse.\r\n                <code data-source=\"33_reference_collapsing\" data-frame-offset=\"3\"></code>\r\n            </li>\r\n            <li data-fade=\"10\"><code>decltype(name)</code> gives the declared type of name.</li>\r\n            <li data-fade=\"11\">Binding or casting an <strong>lvalue</strong> or <strong>xvalue</strong> to the non-reference type, will construct it.</li>\r\n            <li data-fade=\"12\">But casting a <strong>prvalue</strong> to the non-reference type will not.</li>\r\n        </ul>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>prvalue and xvalue</h1>\r\n        <p data-fade=\"0\">Commonly used <strong>xvalues</strong> come from:</p>\r\n        <ol>\r\n            <li data-fade=\"1\">\r\n                Casting to an rvalue reference:\r\n                <code data-source=\"34_xvalue_1\"></code>\r\n            </li>\r\n            <li data-fade=\"2\">\r\n                Function call that returns an rvalue reference:\r\n                <code data-source=\"34_xvalue_2\" data-frame-offset=\"3\"></code>\r\n            </li>\r\n        </ol>\r\n        <p data-fade=\"4\">Most other <strong>rvalues</strong> are <strong>prvalues</strong>.<sup><a href=\"http://en.cppreference.com/w/cpp/language/value_category\">[C++ Ref]</a></sup></p>\r\n        <br />\r\n        <p data-fade=\"8\"><strong>Question:</strong> When would you return an rvalue reference from a function?</p>\r\n        <p data-fade=\"12\">When you want to return both a <em>reference</em> and an <em>rvalue</em>.</p>\r\n        <br />\r\n        <p data-fade=\"16\"><strong>Reminder:</strong> An rvalue reference is still a reference!</p>\r\n        <code data-source=\"34_xvalue_3\" data-fade=\"16\" data-frame-offset=\"17\"></code>\r\n    </article>\r\n\r\n    <article>\r\n        <h1><code>decltype</code> returns</h1>\r\n        <p>If <code>decltype</code>'s argument is in parentheses, it also uses the <em>expression category</em>.</p>\r\n        <code data-source=\"35_decltype\"></code>\r\n    </article>\r\n</section>\r\n",
	"content/40_perfect_forwarding.html": "<section data-footnote=\"Perfect forwarding\">\r\n    <article class=\"intermediate\">\r\n        <h1>Perfect forwarding</h1>\r\n        <h2>To wrap functions correctly.</h2>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Creating components</h1>\r\n        <code data-source=\"40_creating_components\"></code>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Creating components (summary)</h1>\r\n        <p>Takeaways:</p>\r\n        <ul>\r\n            <li data-fade=\"0\"><code>T&&</code> is <em>forwarding reference</em> iif <code>T</code> is deduced type.</li>\r\n            <li data-fade=\"1\">No <strong>cv-qualifiers</strong> allowed on <strong>forwarding reference</strong>!</li>\r\n            <li data-fade=\"2\">If argument is an <strong>lvalue</strong>, the compiler adds <strong>lvalue reference</strong> to the deduced type.<sup><a href=\"http://en.cppreference.com/w/cpp/language/template_argument_deduction#Deduction_from_a_function_call\">[C++ Ref]</a></sup></li>\r\n            <li data-fade=\"3\">Combine with <code>std::forward&lt;T&gt;</code>.</li>\r\n            <li data-fade=\"4\"><em>Universal reference</em> is synonymous with <strong>forwarding reference</strong>.</li>\r\n            <li data-fade=\"5\">Use <code>static_assert</code> to make your error messages more readable.</li>\r\n        </ul>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Return values</h1>\r\n        <code data-source=\"41_return_values\"></code>\r\n        <p data-fade=\"20\">When is this useful? <span data-fade=\"21\">Rarely, but if you need it, it's invaluable.</span></p>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Performance</h1>\r\n        <code data-source=\"42_performance\"></code>\r\n        <p data-fade=\"10\">How often does the vector get copied?</p>\r\n        <p data-fade=\"11\"><em>0</em> copies (and <em>0</em> moves).</p>\r\n        <p data-fade=\"12\"><strong>Guaranteed by standard!</strong></p>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Implementation of <code>std::move</code> and <code>std::forward</code></h1>\r\n        <code data-source=\"43_move_forward\"></code>\r\n        <p data-fade=\"11\">No compiler magic, just plain code.</p>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Summary</h1>\r\n        <p>Summary:</p>\r\n        <ul>\r\n            <li data-fade=\"0\">Perfect forwarding to pass parameters/return values to/from other functions.</li>\r\n            <li data-fade=\"1\">Be aware of forwarding references (<code>T&&</code> where <code>T</code> is deduced).</li>\r\n            <li data-fade=\"2\">Return by value, it gets optimized.</li>\r\n        </ul>\r\n        <p data-fade=\"10\">Guidelines:</p>\r\n        <ul>\r\n            <li data-fade=\"10\">A move operation should leave its source in a valid state.<sup><a href=\"https://isocpp.github.io/CppCoreGuidelines/CppCoreGuidelines#c64-a-move-operation-should-move-and-leave-its-source-in-a-valid-state\">[C.64]</a></sup></li>\r\n            <li data-fade=\"11\">Use <code>std::move</code> when you need to move an object to another scope.<sup><a href=\"https://isocpp.github.io/CppCoreGuidelines/CppCoreGuidelines#es56-write-stdmove-only-when-you-need-to-explicitly-move-an-object-to-another-scope\">[ES.56]</a></sup></li>\r\n            <li data-fade=\"12\">Pass by <code>T&&</code> for parameters that consume the input.<sup><a href=\"https://isocpp.github.io/CppCoreGuidelines/CppCoreGuidelines#f18-for-consume-parameters-pass-by-x-and-stdmove-the-parameter\">[F.18]</a></sup></li>\r\n            <li data-fade=\"13\">Only use forwarding references inside <code>std::forward</code>.<sup><a href=\"https://isocpp.github.io/CppCoreGuidelines/CppCoreGuidelines#f19-for-forward-parameters-pass-by-tp-and-only-stdforward-the-parameter\">[F.19]</a></sup></li>\r\n        </ul>\r\n    </article>\r\n</section>\r\n",
	"content/50_exercises.html": "<section data-footnote=\"Exercises\">\r\n    <article class=\"intermediate\">\r\n        <h1>Exercises</h1>\r\n        <h2>Practice makes perfect.</h2>\r\n    </article>\r\n    \r\n    <article>\r\n        <h1>Expression Category quiz!</h1>\r\n        <p><strong>Goal: </strong> Look at an expression, and determine the expression category.</p>\r\n        <br />\r\n        <ul>\r\n            <li data-fade=\"0\">15 questions.</li>\r\n            <li data-fade=\"1\">Answers immediately visible after submitting.</li>\r\n        </ul>\r\n        <br />\r\n        <p data-fade=\"2\"><strong>Link:</strong> <a href=\"https://goo.gl/forms/bE1bXpzyU5TjiOCc2\">https://goo.gl/forms/bE1bXpzyU5TjiOCc2</a></p>\r\n        <p data-fade=\"3\"><strong>Estimated duration:</strong> 2-5 minutes.</p>\r\n    </article>\r\n\r\n    <article>\r\n        <h1>Exercise 1 - Move operations</h1>\r\n        <p><strong>Goal:</strong> Write copy/move constructor/assignment for <code>BitArray</code> class.</p>\r\n        <br />\r\n        <p data-fade=\"0\">Steps:</p>\r\n        <ul>\r\n            <li data-fade=\"0\">Add declarations in <strong>exercise.hpp</strong> at line 9.</li>\r\n            <li data-fade=\"1\">Add definitions in <strong>exercise.cpp</strong> at line 36.</li>\r\n            <li data-fade=\"2\">Make the 16 unit tests pass.</li>\r\n        </ul>\r\n        <p data-fade=\"3\">Uses <code>TestAllocator::allocate</code><span style=\"font-family: 'Fira Code'\">&nbsp;&nbsp;</span> instead of <code>new unsigned char[]</code>.</p>\r\n        <p data-fade=\"3\">Uses <code>TestAllocator::deallocate</code> instead of <code>delete[]</code>.</p>\r\n        <br />\r\n        <p data-fade=\"4\"><strong>Knowledge:</strong> Default operations, exception safety.</p>\r\n        <p data-fade=\"5\"><strong>Estimated duration:</strong> 5-60 minutes.</p>\r\n    </article>\r\n    \r\n    <article>\r\n        <h1>Exercise 2 - Combined IO streams</h1>\r\n        <p><strong>Goal:</strong> Write a function that appends (<code><<</code>) each of its arguments to a <code>std::stringstream</code>.</p>\r\n        <br />\r\n        <ul>\r\n            <li data-fade=\"0\">Implement the function in <code>make_sstream.hpp</code>.</li>\r\n            <li data-fade=\"1\">Make the 6 unit tests pass.</li>\r\n        </ul>\r\n        <br />\r\n        <p data-fade=\"2\"><strong>Knowledge:</strong> Variadic templates, perfect forwarding.</p>\r\n        <p data-fade=\"3\"><strong>Estimated duration:</strong> 5-20 minutes.</p>\r\n    </article>\r\n</section>\r\n",
	"content/60_end.html": "<section>\r\n    <article class=\"intermediate\">\r\n        <h1>Thanks for attending</h1>\r\n        <h2>Questions?</h2>\r\n    </article>\r\n    <article>\r\n        <h1>Contact</h1>\r\n        <p>Feel free to contact me if you have questions later on.</p>\r\n        <table>\r\n            <tr>\r\n                <th>E-mail</th>\r\n                <td>143042@nhtv.nl</td>\r\n            </tr>\r\n            <tr>\r\n                <th>Slack</th>\r\n                <td>Caspar143042</td>\r\n            </tr>\r\n            <tr>\r\n                <th>Discord</th>\r\n                <td>Aidiakapi#2177</td>\r\n            </tr>\r\n        </table>\r\n\r\n        <p><em>Please give me feedback!</em> <a href=\"https://goo.gl/forms/zVXOH97o1iyB4qIj1\">https://goo.gl/forms/zVXOH97o1iyB4qIj1</a></p>\r\n    </article>\r\n</section>\r\n",
	"content/slides.json": "[\r\n    \"10_introduction\",\r\n    \"20_move_semantics\",\r\n    \"30_references\",\r\n    \"40_perfect_forwarding\",\r\n    \"50_exercises\",\r\n    \"60_end\"\r\n]\r\n",
	"content/code/20_non_copyable.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\"sl\", [1,5], \"sm\", [10,15,25,\"hide\"], [10,31,32,\"hide\"]],\r\n            [\"sl\", [6,9], 14],\r\n            [\"sl\", 10],\r\n            [\"sn\", [10,4,\"What happens?\"]],\r\n            [\"cn\", \"sl\", [11,13]],\r\n            [\"sm\", [13,56,64]],\r\n            [\"sm\", [11,21,69]],\r\n            [\"sm\", [11,71,81]],\r\n            [\"sm\", [12,27,72]],\r\n            [\"sm\", [12,21,26], [12,73,74]],\r\n            [\"cm\", \"sn\", [13,21,\"Signature: T(T const&)\",\"info\"]],\r\n            [\"an\", [13,21,\"What is that called?\"]],\r\n            [\"sn\", [13,21,\"Trying to call the copy constructor.\",\"info\"]],\r\n            [\"cn\"],\r\n            [],\r\n            [],\r\n            [\"hl\", [11,13], \"cm\", [\"hide\"], \"sm\", [10,15,25], [10,31,32]],\r\n            [\"sn\", [10,4,\"Moving a unique_ptr instead transfers ownership. player == nullptr\",\"info\"]],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"struct Player { /* stuff here */ };\",\r\n        \"struct Game\",\r\n        \"{\",\r\n        \"    std::unique_ptr<Player> player;\",\r\n        \"};\",\r\n        \"\",\r\n        \"static void example()\",\r\n        \"{\",\r\n        \"    auto player = std::make_unique<Player>();\",\r\n        \"    Game game{ std::move(player) };\",\r\n        \"    // error C2280: 'std::unique_ptr<Player,std::default_delete<_Ty>>::unique_ptr\",\r\n        \"    //              (const std::unique_ptr<_Ty,std::default_delete<_Ty>> &)'\",\r\n        \"    //              : attempting to reference a deleted function\",\r\n        \"}\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 7, \"#d38569\", 7, \"#cccccc\", 2, \"#91968b\", 17, \"#cccccc\", 2 ],\r\n        [ \"#b98eb2\", 7, \"#d38569\", 4 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#d36971\", 7, \"#cccccc\", 2, \"#d38569\", 10, \"#cccccc\", 1, \"#d38569\", 6, \"#cccccc\", 2, \"plum\", 6, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 2 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 7, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 9, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d38569\", 6, \"#cccccc\", 4 ],\r\n        [ \"#d38569\", 9, \"#cccccc\", 6, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 11 ],\r\n        [ \"#91968b\", 81 ],\r\n        [ \"#91968b\", 76 ],\r\n        [ \"#91968b\", 64 ],\r\n        [ \"#cccccc\", 1 ]\r\n    ]\r\n}",
	"content/code/21_unnecessary_copy.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\"sl\", [1,6], \"sm\", [11,15,25,\"hide\"], [11,32,33,\"hide\"]],\r\n            [\"sl\", [7,10], 12],\r\n            [\"sl\", 11],\r\n            [\"sn\", [11,13,\"What happens to bullets?\"]],\r\n            [\"sn\", [11,13,\"1000 bullets get copied.\",\"info\"]],\r\n            [\"cn\", \"cm\", [\"hide\"], \"sm\", [11,15,25], [11,32,33]],\r\n            [\"cm\", \"sn\", [11,4,\"Moving the vector is basically a pointer-swap.\",\"info\"]],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"struct Bullet { /* bullet data here */ };\",\r\n        \"struct Game\",\r\n        \"{\",\r\n        \"    std::vector<Bullet> bullets;\",\r\n        \"};\",\r\n        \"\",\r\n        \"static void example()\",\r\n        \"{\",\r\n        \"    std::vector<Bullet> bullets;\",\r\n        \"    // Inserts 1000 bullets here\",\r\n        \"    Game game{ std::move(bullets) };\",\r\n        \"}\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 7, \"#d38569\", 7, \"#cccccc\", 2, \"#91968b\", 23, \"#cccccc\", 2 ],\r\n        [ \"#b98eb2\", 7, \"#d38569\", 4 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#d36971\", 7, \"#cccccc\", 2, \"#d38569\", 6, \"#cccccc\", 1, \"#d38569\", 6, \"#cccccc\", 2, \"plum\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 2 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 7, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#d36971\", 7, \"#cccccc\", 2, \"#d38569\", 6, \"#cccccc\", 1, \"#d38569\", 6, \"#cccccc\", 10 ],\r\n        [ \"#91968b\", 32 ],\r\n        [ \"#d38569\", 9, \"#cccccc\", 6, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 12 ],\r\n        [ \"#cccccc\", 1 ]\r\n    ]\r\n}",
	"content/code/22_surface.json": "{\r\n    \"sets\": {\r\n        \"copy_issues\": [\r\n            [\"sl\", [1,2], [67,71]],\r\n            [\"sl\", [3,8]],\r\n            [\"hl\", [5,8], \"sl\", [9,10], 26],\r\n            [\"sl\", [11,16]],\r\n            [\"sl\", [17,20]],\r\n            [\"sm\", [20,38,54], [20,68,69]],\r\n            [\"sn\", [20,38,\"Compiler specific!\",\"warning\"]],\r\n            [\"cm\", \"sn\", [20,8,\"What if the allocation fails?\"]],\r\n            [\"cn\", \"sl\", [21,24]],\r\n            [\"sl\", 25],\r\n            [\"hl\", [5,8], [10,26]],\r\n            [\"sm\", [56,12,26], [56,33,34], \"sl\", [52,58]],\r\n            [\"hl\", [53,58], \"sn\", [71,0,\"Correct?\"]],\r\n            [\"sn\", [71,0,\"The copy constructor/assignment!\",\"info\"]],\r\n            [\"cn\", \"sl\", [27,34]],\r\n            [\"sm\", [28,8,42]],\r\n            [\"sm\", [32,12,24], [32,62,63]],\r\n            [\"cm\"],\r\n            [\"sl\", [35,40]],\r\n            [\"sm\", [27,4,31], [37,8,24]],\r\n            [\"sm\", [38,8,19]],\r\n            [\"sm\", [39,8,21]],\r\n            [\"cm\"],\r\n            [\"hl\", [28,34], [36,40], \"sl\", [60,65]],\r\n            [\"hl\", [61,65], \"sl\", [72,76]],\r\n            [\"hl\", [74,76], \"sn\", [73,0,\"Correct?\"]],\r\n            [\"sn\", [73,0,\"Yes, let's use it.\",\"info\"]],\r\n            [\"cn\", \"hl\", [2,73], \"sl\", 78],\r\n            [\"sl\", [79,81],83],\r\n            [\"sl\", 82],\r\n            [\"sm\", [82,21,30], \"sn\", [82,21,\"What happens?\"]],\r\n            [\"sn\", [82,21,\"32768 pixels get copied\",\"info\"]],\r\n            [\"cm\", \"sn\", [83,0,\"Fixes?\"]],\r\n            [\"cn\", \"sl\", [84,88], 90],\r\n            [\"sl\", 89],\r\n            [\"sn\", [90,0,\"Problem?\"]],\r\n            [\"sn\",\r\n                [90,0,\"1) Requires breaking change on API.\",\"info\"],\r\n                [90,0,\"2) Extra allocation.\",\"info\"],\r\n                [90,0,\"3) Every access to image requires extra indirection.\",\"info\"]\r\n            ],\r\n            [\"cn\", \"sl\", [91,96]],\r\n            [\"sm\", [95,23,39]],\r\n            [\"sm\", [95,21,41], \"sn\", [95,21,\"What happens?\"]],\r\n            [\"sn\", [95,21,\"32768 pixels get copied\",\"info\"]],\r\n            [\"an\", [95,21,\"but that can be changed (suspense)\",\"info\"]],\r\n            null\r\n        ],\r\n        \"move_semantics\": [\r\n            [\"sl\", [1,1], 78, [92,96], \"sn\", [95,4,\"What would you like to happen?\"]],\r\n            [\"an\", [95,4,\"1) Player must have the correct image data.\",\"info\"]],\r\n            [\"an\", [95,4,\"2) No copying the image data.\",\"info\"]],\r\n            [\"an\", [95,4,\"3) 0 allocations, 0 frees.\",\"info\"]],\r\n            [\"an\", [95,4,\"4) 'image' should now be \\\"empty\\\".\",\"info\"]],\r\n            [\"an\", [95,4,\"How to make it happen?\"]],\r\n            [\"cn\", \"sl\", [2,3], 41, 71],\r\n            [\"sn\", [41,4,\"Move constructor, signature: T(T&&) noexcept\",\"info\"]],\r\n            [\"sm\", [41,27,35], \"sn\", [41,17,\"Important noexcept!\",\"info\"]],\r\n            [\"sm\", [41,19,21], \"sn\", [41,11,\"rvalue reference\",\"info\"]],\r\n            [\"cn\", \"cm\", \"sl\", [42,45]],\r\n            [\"sm\", [42,8,17], \"sl\", [4,8]],\r\n            [\"sm\", [44,8,18], \"hl\", [4,8], \"sl\", [60,65]],\r\n            [\"hl\", [60,65], \"cm\", \"sn\", [43,8,\"*this === player.image           (is empty)\",\"info\"], [43,8,\"rhs   === local variable 'image' (correct data)\",\"info\"]],\r\n            [\"an\", [44,8,\"*this === player.image           (correct data)\",\"info\"], [44,8,\"rhs   === local variable 'image' (is empty)\",\"info\"]],\r\n            [\"cn\", \"hl\", [92,96], \"sl\", [98,101], 103],\r\n            [\"sl\", 102],\r\n            [\"sm\", [102,16,35], \"sn\", [102,4,\"What does this do?\"]],\r\n            [\"sn\", [102,4,\"Calls the copy assignment operator...\",\"info\"]],\r\n            [\"an\", [102,4,\"unless we implement a move assignment operator.\",\"info\"]],\r\n            [\"cn\", \"cm\", \"sl\", 46],\r\n            [\"sn\", [46,4,\"Move assignment, signature: T& operator=(T&&) noexcept\",\"info\"]],\r\n            [\"sm\", [46,30,32], [46,38,46]],\r\n            [\"cm\", \"cn\", \"sl\", 47, [50,51]],\r\n            [\"sl\", [48,49]],\r\n            [\"sm\", [41,4,35], [48,8,35]],\r\n            [\"cm\", \"sn\", [48,8,\"Is std::move here necessary?\"]],\r\n            [\"sn\", [48,8,\"Absolutely! Without it, it would call the copy constructor!\",\"warning\"]],\r\n            [\"cn\"],\r\n            null\r\n        ],\r\n        \"expression_categories\": [\r\n            [\"sl\", [9,16], [21,26]],\r\n            [\"sm\", [11,12,17]],\r\n            [\"sn\", [11,12,\"lvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [11,26,32]],\r\n            [\"sn\", [11,26,\"lvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [11,21,22], [11,36,37], [13,31,32], [25,47,48]],\r\n            [\"sn\", [11,21,\"rvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [11,12,22]],\r\n            [\"sn\", [11,12,\"rvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [11,12,37]],\r\n            [\"sn\", [11,12,\"rvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [13,12,18], [13,21,28], [14,12,19], [21,12,19], [25,20,27], [25,29,35], [25,38,45]],\r\n            [\"sn\", [13,12,\"lvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [13,21,32]],\r\n            [\"sn\", [13,21,\"lvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [13,12,32], [14,12,29]],\r\n            [\"sn\", [14,12,\"lvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [14,22,29], [21,23,30]],\r\n            [\"sn\", [14,22,\"rvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [21,12,30]],\r\n            [\"sn\", [21,12,\"rvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [23,18,34]],\r\n            [\"sn\", [23,18,\"rvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [23,12,34]],\r\n            [\"sn\", [23,12,\"Is throw even an expression?\"]],\r\n            [\"sn\", [23,12,\"Yes it is! Its type: void\",\"info\"]],\r\n            [\"sn\", [23,12,\"rvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [25,8,19]],\r\n            [\"sn\", [25,8,\"lvalue\",\"info\"]],\r\n            [\"cn\", \"sm\", [25,8,49]],\r\n            [\"sn\", [25,8,\"rvalue\",\"info\"]],\r\n            [\"cn\", \"cm\"],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"class Surface\",\r\n        \"{\",\r\n        \"public:\",\r\n        \"    Surface() noexcept :\",\r\n        \"        width_ { 0 },\",\r\n        \"        height_{ 0 },\",\r\n        \"        buffer_{ nullptr }\",\r\n        \"    { }\",\r\n        \"    Surface(int32_t const width, int32_t const height)\",\r\n        \"    {\",\r\n        \"        if (width <= 0 || height <= 0)\",\r\n        \"        {\",\r\n        \"            width_ = height_ = 0;\",\r\n        \"            buffer_ = nullptr;\",\r\n        \"            return;\",\r\n        \"        }\",\r\n        \"        size_t const byte_count = width * height * sizeof(Pixel);\",\r\n        \"        width_  = width;\",\r\n        \"        height_ = height;\",\r\n        \"        buffer_ = static_cast<Pixel*>(_aligned_malloc(byte_count, 16));\",\r\n        \"        if (buffer_ == nullptr)\",\r\n        \"        {\",\r\n        \"            throw std::bad_alloc{};\",\r\n        \"        }\",\r\n        \"        std::fill_n(buffer_, width_ * height_, 0);\",\r\n        \"    }\",\r\n        \"    Surface(Surface const& rhs) :\",\r\n        \"        Surface{ rhs.width_, rhs.height_ }\",\r\n        \"    {\",\r\n        \"        if (buffer_ != nullptr)\",\r\n        \"        {\",\r\n        \"            std::copy_n(rhs.buffer_, width_ * height_, buffer_);\",\r\n        \"        }\",\r\n        \"    }\",\r\n        \"    Surface& operator=(Surface const& rhs)\",\r\n        \"    {\",\r\n        \"        auto copy = rhs;\",\r\n        \"        swap(copy);\",\r\n        \"        return *this;\",\r\n        \"    }\",\r\n        \"    Surface(Surface&& rhs) noexcept :\",\r\n        \"        Surface{}\",\r\n        \"    {\",\r\n        \"        swap(rhs);\",\r\n        \"    }\",\r\n        \"    Surface& operator=(Surface&& rhs) noexcept\",\r\n        \"    {\",\r\n        \"        auto move = std::move(rhs);\",\r\n        \"        swap(move);\",\r\n        \"        return *this;\",\r\n        \"    }\",\r\n        \"    ~Surface() noexcept\",\r\n        \"    {\",\r\n        \"        if (buffer_ != nullptr)\",\r\n        \"        {\",\r\n        \"            _aligned_free(buffer_);\",\r\n        \"        }\",\r\n        \"    }\",\r\n        \"\",\r\n        \"    void swap(Surface& rhs) noexcept\",\r\n        \"    {\",\r\n        \"        std::swap(buffer_, rhs.buffer_);\",\r\n        \"        std::swap(width_ , rhs.width_ );\",\r\n        \"        std::swap(height_, rhs.height_);\",\r\n        \"    }\",\r\n        \"\",\r\n        \"private:\",\r\n        \"    int32_t width_;\",\r\n        \"    int32_t height_;\",\r\n        \"    Pixel*  buffer_;\",\r\n        \"};\",\r\n        \"\",\r\n        \"inline void swap(Surface& lhs, Surface& rhs) noexcept\",\r\n        \"{\",\r\n        \"    lhs.swap(rhs);\",\r\n        \"}\",\r\n        \"\",\r\n        \"struct GameObject { Surface image; };\",\r\n        \"static void example_copy()\",\r\n        \"{\",\r\n        \"    Surface image{ 128, 256 };\",\r\n        \"    GameObject player{ image };\",\r\n        \"}\",\r\n        \"\",\r\n        \"struct GameObjectUptr { std::unique_ptr<Surface> image; };\",\r\n        \"static void example_uptr()\",\r\n        \"{\",\r\n        \"    auto image = std::make_unique<Surface>(128, 256);\",\r\n        \"    GameObjectUptr player{ std::move(image) };\",\r\n        \"}\",\r\n        \"\",\r\n        \"static void example_move()\",\r\n        \"{\",\r\n        \"    Surface image{ 128, 256 };\",\r\n        \"    GameObject player{ std::move(image) };\",\r\n        \"}\",\r\n        \"\",\r\n        \"static void example_move_assign()\",\r\n        \"{\",\r\n        \"    GameObject player;\",\r\n        \"    Surface image{ 128, 256 };\",\r\n        \"    player.image = std::move(image);\",\r\n        \"}\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 6, \"#d38569\", 7 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 6, \"#cccccc\", 1 ],\r\n        [ \"#c8c8c8\", 11, \"#cccccc\", 3, \"#b98eb2\", 9, \"#cccccc\", 1 ],\r\n        [ \"#e0c0c0\", 15, \"#cccccc\", 2, \"#819dc2\", 2, \"#cccccc\", 2 ],\r\n        [ \"#e0c0c0\", 15, \"#cccccc\", 2, \"#819dc2\", 2, \"#cccccc\", 2 ],\r\n        [ \"#e0c0c0\", 15, \"#cccccc\", 2, \"#b98eb2\", 8, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 7 ],\r\n        [ \"#c8c8c8\", 11, \"#cccccc\", 1, \"#d38569\", 8, \"#b98eb2\", 6, \"#cccccc\", 7, \"#d38569\", 8, \"#b98eb2\", 6, \"#cccccc\", 7 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#b98eb2\", 11, \"#cccccc\", 10, \"#819dc2\", 2, \"#cccccc\", 13, \"#819dc2\", 1, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 9 ],\r\n        [ \"#e0c0c0\", 19, \"#cccccc\", 2, \"#e0c0c0\", 8, \"#cccccc\", 2, \"#819dc2\", 1, \"#cccccc\", 1 ],\r\n        [ \"#e0c0c0\", 20, \"#cccccc\", 2, \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 18, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 9 ],\r\n        [ \"#d38569\", 15, \"#b98eb2\", 6, \"#cccccc\", 30, \"#b98eb2\", 6, \"#cccccc\", 1, \"#d38569\", 5, \"#cccccc\", 2 ],\r\n        [ \"#e0c0c0\", 16, \"#cccccc\", 8 ],\r\n        [ \"#e0c0c0\", 16, \"#cccccc\", 9 ],\r\n        [ \"#e0c0c0\", 16, \"#cccccc\", 2, \"#b98eb2\", 11, \"#cccccc\", 1, \"#d38569\", 5, \"#cccccc\", 3, \"#d38569\", 15, \"#cccccc\", 13, \"#819dc2\", 2, \"#cccccc\", 3 ],\r\n        [ \"#b98eb2\", 11, \"#cccccc\", 1, \"#e0c0c0\", 8, \"#cccccc\", 3, \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 9 ],\r\n        [ \"#b98eb2\", 18, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 9, \"#cccccc\", 3 ],\r\n        [ \"#cccccc\", 9 ],\r\n        [ \"#d36971\", 11, \"#cccccc\", 2, \"#d38569\", 6, \"#cccccc\", 1, \"#e0c0c0\", 7, \"#cccccc\", 2, \"#e0c0c0\", 7, \"#cccccc\", 2, \"#e0c0c0\", 7, \"#cccccc\", 2, \"#819dc2\", 1, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#c8c8c8\", 11, \"#cccccc\", 1, \"#d38569\", 8, \"#b98eb2\", 5, \"#cccccc\", 8 ],\r\n        [ \"#d38569\", 15, \"#cccccc\", 6, \"#e0c0c0\", 6, \"#cccccc\", 6, \"#e0c0c0\", 8, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#b98eb2\", 11, \"#cccccc\", 1, \"#e0c0c0\", 8, \"#cccccc\", 3, \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 9 ],\r\n        [ \"#d36971\", 15, \"#cccccc\", 2, \"#d38569\", 6, \"#cccccc\", 5, \"#e0c0c0\", 7, \"#cccccc\", 2, \"#e0c0c0\", 7, \"#cccccc\", 2, \"#e0c0c0\", 7, \"#cccccc\", 2, \"#e0c0c0\", 7, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 9 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#d38569\", 11, \"#cccccc\", 2, \"#b4b4b4\", 9, \"#cccccc\", 1, \"#d38569\", 8, \"#b98eb2\", 5, \"#cccccc\", 6 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#b98eb2\", 13, \"#cccccc\", 11 ],\r\n        [ \"#c8c8c8\", 12, \"#cccccc\", 7 ],\r\n        [ \"#b98eb2\", 15, \"#cccccc\", 1, \"#b98eb2\", 4, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#c8c8c8\", 11, \"#cccccc\", 1, \"#d38569\", 7, \"#cccccc\", 8, \"#b98eb2\", 9, \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 15, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#c8c8c8\", 12, \"#cccccc\", 6 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#d38569\", 11, \"#cccccc\", 2, \"#b4b4b4\", 9, \"#cccccc\", 1, \"#d38569\", 7, \"#cccccc\", 8, \"#b98eb2\", 8 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#b98eb2\", 13, \"#cccccc\", 7, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 6 ],\r\n        [ \"#c8c8c8\", 12, \"#cccccc\", 7 ],\r\n        [ \"#b98eb2\", 15, \"#cccccc\", 1, \"#b98eb2\", 4, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#c8c8c8\", 12, \"#cccccc\", 3, \"#b98eb2\", 8 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#b98eb2\", 11, \"#cccccc\", 1, \"#e0c0c0\", 8, \"#cccccc\", 3, \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 9 ],\r\n        [ \"#d38569\", 25, \"#cccccc\", 1, \"#e0c0c0\", 7, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 9 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 9, \"#c8c8c8\", 4, \"#cccccc\", 1, \"#d38569\", 7, \"#cccccc\", 7, \"#b98eb2\", 8 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#d36971\", 11, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 1, \"#e0c0c0\", 7, \"#cccccc\", 6, \"#e0c0c0\", 7, \"#cccccc\", 2 ],\r\n        [ \"#d36971\", 11, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 1, \"#e0c0c0\", 7, \"#cccccc\", 6, \"#e0c0c0\", 7, \"#cccccc\", 2 ],\r\n        [ \"#d36971\", 11, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 1, \"#e0c0c0\", 7, \"#cccccc\", 6, \"#e0c0c0\", 7, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 12, \"#e0c0c0\", 6, \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 12, \"#e0c0c0\", 7, \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 9, \"#cccccc\", 3, \"#e0c0c0\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 2 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 4, \"#cccccc\", 1, \"#d38569\", 7, \"#cccccc\", 7, \"#d38569\", 7, \"#cccccc\", 7, \"#b98eb2\", 8 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 8, \"#c8c8c8\", 4, \"#cccccc\", 6 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 7, \"#d38569\", 11, \"#cccccc\", 2, \"#d38569\", 8, \"plum\", 5, \"#cccccc\", 4 ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 12, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 12, \"#cccccc\", 7, \"#819dc2\", 3, \"#cccccc\", 2, \"#819dc2\", 4, \"#cccccc\", 2 ],\r\n        [ \"#d38569\", 15, \"#cccccc\", 16 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 7, \"#d38569\", 15, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 10, \"#cccccc\", 1, \"#d38569\", 7, \"#cccccc\", 2, \"plum\", 5, \"#cccccc\", 4 ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 12, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 8, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d38569\", 7, \"#cccccc\", 2, \"#819dc2\", 3, \"#cccccc\", 2, \"#819dc2\", 3, \"#cccccc\", 2 ],\r\n        [ \"#d38569\", 19, \"#cccccc\", 8, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 10 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 12, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 12, \"#cccccc\", 7, \"#819dc2\", 3, \"#cccccc\", 2, \"#819dc2\", 4, \"#cccccc\", 2 ],\r\n        [ \"#d38569\", 15, \"#cccccc\", 8, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 10 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 19, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 15, \"#cccccc\", 7 ],\r\n        [ \"#d38569\", 12, \"#cccccc\", 7, \"#819dc2\", 3, \"#cccccc\", 2, \"#819dc2\", 4, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 11, \"plum\", 6, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 8 ],\r\n        [ \"#cccccc\", 1 ]\r\n    ]\r\n}",
	"content/code/23_default_operations.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\"sl\", [1,3], [27,30], \"sm\", [4,36,62,\"hide\"]],\r\n            [\"sl\", 4],\r\n            [\"cm\", [\"hide\"]],\r\n            [\"hl\", 4, \"sl\", [5,8]],\r\n            [\"hl\", [5,8], \"sl\", [9,14]],\r\n            [\"hl\", [9,14], \"sl\", [15,18]],\r\n            [\"hl\", [15,18], \"sl\", [19,24]],\r\n            [\"hl\", [19,24], \"sl\", 25],\r\n            [\"hl\", [1,30]],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"class DefaultOperations\",\r\n        \"{\",\r\n        \"public:\",\r\n        \"    DefaultOperations() noexcept {} // No value initialization\",\r\n        \"    DefaultOperations(DefaultOperations const& rhs) :\",\r\n        \"        a{ rhs.a },\",\r\n        \"        b{ rhs.b }\",\r\n        \"    { }\",\r\n        \"    DefaultOperations& operator=(DefaultOperations const& rhs)\",\r\n        \"    {\",\r\n        \"        a = rhs.a;\",\r\n        \"        b = rhs.b;\",\r\n        \"        return *this;\",\r\n        \"    }\",\r\n        \"    DefaultOperations(DefaultOperations&& rhs) noexcept :\",\r\n        \"        a{ std::move(rhs.a) },\",\r\n        \"        b{ std::move(rhs.b) }\",\r\n        \"    { }\",\r\n        \"    DefaultOperations& operator=(DefaultOperations&& rhs) noexcept\",\r\n        \"    {\",\r\n        \"        a = std::move(rhs.a);\",\r\n        \"        b = std::move(rhs.b);\",\r\n        \"        return *this;\",\r\n        \"    }\",\r\n        \"    ~DefaultOperations() noexcept {}\",\r\n        \"\",\r\n        \"private:\",\r\n        \"    int a;\",\r\n        \"    std::vector<int> b;\",\r\n        \"};\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 6, \"#d38569\", 17 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 6, \"#cccccc\", 1 ],\r\n        [ \"#c8c8c8\", 21, \"#cccccc\", 3, \"#b98eb2\", 9, \"#cccccc\", 3, \"#91968b\", 26 ],\r\n        [ \"#c8c8c8\", 21, \"#cccccc\", 1, \"#d38569\", 18, \"#b98eb2\", 5, \"#cccccc\", 8 ],\r\n        [ \"#e0c0c0\", 9, \"#cccccc\", 6, \"#e0c0c0\", 2, \"#cccccc\", 2 ],\r\n        [ \"#e0c0c0\", 9, \"#cccccc\", 6, \"#e0c0c0\", 2, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 7 ],\r\n        [ \"#d38569\", 21, \"#cccccc\", 2, \"#b4b4b4\", 9, \"#cccccc\", 1, \"#d38569\", 18, \"#b98eb2\", 5, \"#cccccc\", 6 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#e0c0c0\", 10, \"#cccccc\", 6, \"#e0c0c0\", 1, \"#cccccc\", 1 ],\r\n        [ \"#e0c0c0\", 10, \"#cccccc\", 6, \"#e0c0c0\", 1, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 15, \"#cccccc\", 1, \"#b98eb2\", 4, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#c8c8c8\", 21, \"#cccccc\", 1, \"#d38569\", 17, \"#cccccc\", 8, \"#b98eb2\", 9, \"#cccccc\", 1 ],\r\n        [ \"#e0c0c0\", 9, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 5, \"#e0c0c0\", 1, \"#cccccc\", 4 ],\r\n        [ \"#e0c0c0\", 9, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 5, \"#e0c0c0\", 1, \"#cccccc\", 3 ],\r\n        [ \"#cccccc\", 7 ],\r\n        [ \"#d38569\", 21, \"#cccccc\", 2, \"#b4b4b4\", 9, \"#cccccc\", 1, \"#d38569\", 17, \"#cccccc\", 8, \"#b98eb2\", 8 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#e0c0c0\", 10, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 5, \"#e0c0c0\", 1, \"#cccccc\", 2 ],\r\n        [ \"#e0c0c0\", 10, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 5, \"#e0c0c0\", 1, \"#cccccc\", 2 ],\r\n        [ \"#b98eb2\", 15, \"#cccccc\", 1, \"#b98eb2\", 4, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#c8c8c8\", 22, \"#cccccc\", 3, \"#b98eb2\", 9, \"#cccccc\", 2 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 8, \"#e0c0c0\", 1, \"#cccccc\", 1 ],\r\n        [ \"#d36971\", 7, \"#cccccc\", 2, \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 2, \"#e0c0c0\", 1, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 2 ]\r\n    ]\r\n}",
	"content/code/23_default_operations_old.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\r\n                \"sl\", [1,5], 9, 15, 19, 25, 31,\r\n                \"sm\",\r\n                [4,35,48,\"hide\"], [5,63,76,\"hide\"], [9,63,76,\"hide\"], [15,58,71,\"hide\"], [19,58,71,\"hide\"], [25,35,48,\"hide\"],\r\n                [35,34,59,\"hide\"], [36,34,65,\"hide\"], [37,34,69,\"hide\"], [38,34,57,\"hide\"],\r\n                [40,17,36,\"hide\"], [41,17,36,\"hide\"], [42,17,35,\"hide\"],\r\n                [44,28,47,\"hide\"], [45,28,47,\"hide\"], [46,28,46,\"hide\"]\r\n            ],\r\n            [\"sn\", [4,4, \"Default constructor\",\"info\"]],\r\n            [\"sn\", [5,4, \"Copy constructor\",\"info\"]],\r\n            [\"sn\", [9,4, \"Copy assignment\",\"info\"]],\r\n            [\"sn\", [15,4,\"Move constructor\",\"info\"]],\r\n            [\"sn\", [19,4,\"Move assignment\",\"info\"]],\r\n            [\"sn\", [25,4,\"Destructor\",\"info\"]],\r\n            [\r\n                \"sn\",\r\n                [31,0,\"Default constructor <- T()\",\"info\"],\r\n                [31,0,\"Copy constructor    <- T(T const&)\",\"info\"],\r\n                [31,0,\"Copy assignment     <- T& operator=(T const&)\",\"info\"],\r\n                [31,0,\"Move constructor    <- T(T&&)\",\"info\"],\r\n                [31,0,\"Move assignment     <- T& operator=(T&&)\",\"info\"],\r\n                [31,0,\"Destructor          <- ~T()\",\"info\"]\r\n            ],\r\n            [\"cn\", \"hl\", [5,25], \"sl\", [28,30], [32,35], 47],\r\n            [\"sn\", [35,4,\"Value of a, value of b? Name of this initialization type?\"]],\r\n            [\"sn\", [35,4,\"a === undefined  &  b === empty vector\",\"info\"], \"cm\", [\"hide\"], \"sm\",\r\n                [4,35,48,\"hide\"], [5,63,76,\"hide\"], [9,63,76,\"hide\"], [15,58,71,\"hide\"], [19,58,71,\"hide\"], [25,35,48,\"hide\"],\r\n                [36,34,65,\"hide\"], [37,34,69,\"hide\"], [38,34,57,\"hide\"],\r\n                [40,17,36,\"hide\"], [41,17,36,\"hide\"], [42,17,35,\"hide\"],\r\n                [44,28,47,\"hide\"], [45,28,47,\"hide\"], [46,28,46,\"hide\"]\r\n            ],\r\n            [\"sl\", 36, \"sn\", [36,4,\"Value of a, value of b? Name of this initialization type?\"]],\r\n            [\"sn\", [36,4,\"a === 0  &  b === empty vector\",\"info\"], \"cm\", [\"hide\"], \"sm\",\r\n                [4,35,48,\"hide\"], [5,63,76,\"hide\"], [9,63,76,\"hide\"], [15,58,71,\"hide\"], [19,58,71,\"hide\"], [25,35,48,\"hide\"],\r\n                [37,34,69,\"hide\"], [38,34,57,\"hide\"],\r\n                [40,17,36,\"hide\"], [41,17,36,\"hide\"], [42,17,35,\"hide\"],\r\n                [44,28,47,\"hide\"], [45,28,47,\"hide\"], [46,28,46,\"hide\"]\r\n            ],\r\n            [\"sl\", 37, \"sn\", [37,4,\"Value of a, value of b? Name of this initialization type?\"]],\r\n            [\"sn\", [37,4,\"a === 0  &  b === empty vector\",\"info\"], \"cm\", [\"hide\"], \"sm\",\r\n                [4,35,48,\"hide\"], [5,63,76,\"hide\"], [9,63,76,\"hide\"], [15,58,71,\"hide\"], [19,58,71,\"hide\"], [25,35,48,\"hide\"],\r\n                [38,34,57,\"hide\"],\r\n                [40,17,36,\"hide\"], [41,17,36,\"hide\"], [42,17,35,\"hide\"],\r\n                [44,28,47,\"hide\"], [45,28,47,\"hide\"], [46,28,46,\"hide\"]\r\n            ],\r\n            [\"sl\", 38, \"sn\", [38,4,\"And this?\"]],\r\n            [\"sn\", [38,4,\"Forward declares a function named d which returns a DefaultOperations.\",\"info\"], \"cm\", [\"hide\"], \"sm\",\r\n                [4,35,48,\"hide\"], [5,63,76,\"hide\"], [9,63,76,\"hide\"], [15,58,71,\"hide\"], [19,58,71,\"hide\"], [25,35,48,\"hide\"],\r\n                [40,17,36,\"hide\"], [41,17,36,\"hide\"], [42,17,35,\"hide\"],\r\n                [44,28,47,\"hide\"], [45,28,47,\"hide\"], [46,28,46,\"hide\"]\r\n            ],\r\n            [\"cn\", \"hl\", 4, [36,38], \"sl\", 5, 9, 40],\r\n            [\"sn\", [40,4,\"Calls?\"]],\r\n            [\"sn\", [40,4,\"Copy constructor.\",\"info\"], \"cm\", [\"hide\"], \"sm\",\r\n                [4,35,48,\"hide\"], [5,63,76,\"hide\"], [9,63,76,\"hide\"], [15,58,71,\"hide\"], [19,58,71,\"hide\"], [25,35,48,\"hide\"],\r\n                [41,17,36,\"hide\"], [42,17,35,\"hide\"],\r\n                [44,28,47,\"hide\"], [45,28,47,\"hide\"], [46,28,46,\"hide\"]\r\n            ],\r\n            [\"sl\", 41, \"sn\", [41,4,\"Calls?\"]],\r\n            [\"sn\", [41,4,\"Copy constructor.\",\"info\"], \"cm\", [\"hide\"], \"sm\",\r\n                [4,35,48,\"hide\"], [5,63,76,\"hide\"], [9,63,76,\"hide\"], [15,58,71,\"hide\"], [19,58,71,\"hide\"], [25,35,48,\"hide\"],\r\n                [42,17,35,\"hide\"],\r\n                [44,28,47,\"hide\"], [45,28,47,\"hide\"], [46,28,46,\"hide\"]\r\n            ],\r\n            [\"sl\", 42, \"sn\", [42,4,\"Calls?\"]],\r\n            [\"sn\", [42,4,\"Copy assignment.\",\"info\"], \"cm\", [\"hide\"], \"sm\",\r\n                [4,35,48,\"hide\"], [5,63,76,\"hide\"], [9,63,76,\"hide\"], [15,58,71,\"hide\"], [19,58,71,\"hide\"], [25,35,48,\"hide\"],\r\n                [44,28,47,\"hide\"], [45,28,47,\"hide\"], [46,28,46,\"hide\"]\r\n            ],\r\n            [\"cn\", \"hl\", 5, 9, \"sl\", 15, 19, [43,44]],\r\n            [\"sn\", [44,4,\"Calls?\"]],\r\n            [\"sn\", [44,4,\"Move constructor.\",\"info\"], \"cm\", [\"hide\"], \"sm\",\r\n                [4,35,48,\"hide\"], [5,63,76,\"hide\"], [9,63,76,\"hide\"], [15,58,71,\"hide\"], [19,58,71,\"hide\"], [25,35,48,\"hide\"],\r\n                [45,28,47,\"hide\"], [46,28,46,\"hide\"]\r\n            ],\r\n            [\"sl\", 45, \"sn\", [45,4,\"Calls?\"]],\r\n            [\"sn\", [45,4,\"Move constructor.\",\"info\"], \"cm\", [\"hide\"], \"sm\",\r\n                [4,35,48,\"hide\"], [5,63,76,\"hide\"], [9,63,76,\"hide\"], [15,58,71,\"hide\"], [19,58,71,\"hide\"], [25,35,48,\"hide\"],\r\n                [46,28,46,\"hide\"]\r\n            ],\r\n            [\"sl\", 46, \"sn\", [46,4,\"Calls?\"]],\r\n            [\"sn\", [46,4,\"Move assignment.\",\"info\"], \"cm\", [\"hide\"], \"sm\",\r\n                [4,35,48,\"hide\"], [5,63,76,\"hide\"], [9,63,76,\"hide\"], [15,58,71,\"hide\"], [19,58,71,\"hide\"], [25,35,48,\"hide\"]\r\n            ],\r\n            [\"cn\", \"hl\", 15, 19, [32,47]],\r\n            [],\r\n            [],\r\n            [],\r\n            [],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"class DefaultOperations\",\r\n        \"{\",\r\n        \"public:\",\r\n        \"    DefaultOperations           () noexcept(...)                             = default;\",\r\n        \"    DefaultOperations           (DefaultOperations const& rhs) noexcept(...) = default;\",\r\n        \"        : a{rhs.a},\",\r\n        \"          b{rhs.b}\",\r\n        \"    {}\",\r\n        \"    DefaultOperations& operator=(DefaultOperations const& rhs) noexcept(...) = default;\",\r\n        \"    {\",\r\n        \"        a = rhs.a;\",\r\n        \"        b = rhs.b;\",\r\n        \"        return *this;\",\r\n        \"    }\",\r\n        \"    DefaultOperations           (DefaultOperations&& rhs) noexcept(...)      = default;\",\r\n        \"        : a{std::move(rhs.a)},\",\r\n        \"          b{std::move(rhs.b)}\",\r\n        \"    {}\",\r\n        \"    DefaultOperations& operator=(DefaultOperations&& rhs) noexcept(...)      = default;\",\r\n        \"    {\",\r\n        \"        a = std::move(rhs.a);\",\r\n        \"        b = std::move(rhs.b);\",\r\n        \"        return *this;\",\r\n        \"    }\",\r\n        \"    ~DefaultOperations          () noexcept(...)                             = default;\",\r\n        \"    {}\",\r\n        \"\",\r\n        \"private:\",\r\n        \"    int a;\",\r\n        \"    std::vector<int> b;\",\r\n        \"};\",\r\n        \"\",\r\n        \"static void example()\",\r\n        \"{\",\r\n        \"    DefaultOperations a;          // default initialization\",\r\n        \"    DefaultOperations b{};        // value initialization (C++11)\",\r\n        \"    auto c = DefaultOperations(); // value initialization (pre C++11)\",\r\n        \"    DefaultOperations d();        // function declaration\",\r\n        \"\",\r\n        \"    auto e{ a }; // copy constructor\",\r\n        \"    auto f = a;  // copy constructor\",\r\n        \"         f = a;  // copy assignment\",\r\n        \"\",\r\n        \"    auto g{ std::move(a) }; // move constructor\",\r\n        \"    auto h = std::move(a);  // move constructor\",\r\n        \"         h = std::move(a);  // move assignment\",\r\n        \"}\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 6, \"#d38569\", 17 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 6, \"#cccccc\", 1 ],\r\n        [ \"#c8c8c8\", 32, \"#cccccc\", 3, \"#b98eb2\", 8, \"#cccccc\", 36, \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 33, \"#d38569\", 18, \"#b98eb2\", 5, \"#cccccc\", 7, \"#b98eb2\", 8, \"#cccccc\", 8, \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 10, \"#ff3333\", 1, \"#cccccc\", 1, \"#bbbbbb\", 3, \"#cccccc\", 1, \"#bbbbbb\", 1, \"#cccccc\", 2 ],\r\n        [ \"#ff3333\", 11, \"#cccccc\", 1, \"#bbbbbb\", 3, \"#cccccc\", 1, \"#bbbbbb\", 1, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 6 ],\r\n        [ \"#d38569\", 21, \"#cccccc\", 2, \"#b98eb2\", 8, \"#cccccc\", 2, \"#d38569\", 18, \"#b98eb2\", 5, \"#cccccc\", 7, \"#b98eb2\", 8, \"#cccccc\", 8, \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#bbbbbb\", 10, \"#cccccc\", 2, \"#bbbbbb\", 3, \"#cccccc\", 1, \"#bbbbbb\", 1, \"#cccccc\", 1 ],\r\n        [ \"#bbbbbb\", 10, \"#cccccc\", 2, \"#bbbbbb\", 3, \"#cccccc\", 1, \"#bbbbbb\", 1, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 15, \"#cccccc\", 1, \"#b98eb2\", 4, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#bbbbbb\", 32, \"#cccccc\", 1, \"#d38569\", 17, \"#cccccc\", 8, \"#b98eb2\", 8, \"#cccccc\", 13, \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 10, \"#ff3333\", 1, \"#cccccc\", 1, \"#bbbbbb\", 3, \"#cccccc\", 2, \"#bbbbbb\", 4, \"#cccccc\", 1, \"#bbbbbb\", 3, \"#cccccc\", 1, \"#bbbbbb\", 1, \"#cccccc\", 3 ],\r\n        [ \"#ff3333\", 11, \"#cccccc\", 1, \"#bbbbbb\", 3, \"#cccccc\", 2, \"#bbbbbb\", 4, \"#cccccc\", 1, \"#bbbbbb\", 3, \"#cccccc\", 1, \"#bbbbbb\", 1, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 6 ],\r\n        [ \"#d38569\", 21, \"#cccccc\", 2, \"#b98eb2\", 8, \"#cccccc\", 2, \"#d38569\", 17, \"#cccccc\", 8, \"#b98eb2\", 8, \"#cccccc\", 13, \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#bbbbbb\", 10, \"#cccccc\", 2, \"#bbbbbb\", 3, \"#cccccc\", 2, \"#bbbbbb\", 4, \"#cccccc\", 1, \"#bbbbbb\", 3, \"#cccccc\", 1, \"#bbbbbb\", 1, \"#cccccc\", 2 ],\r\n        [ \"#bbbbbb\", 10, \"#cccccc\", 2, \"#bbbbbb\", 3, \"#cccccc\", 2, \"#bbbbbb\", 4, \"#cccccc\", 1, \"#bbbbbb\", 3, \"#cccccc\", 1, \"#bbbbbb\", 1, \"#cccccc\", 2 ],\r\n        [ \"#b98eb2\", 15, \"#cccccc\", 1, \"#b98eb2\", 4, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#cccccc\", 5, \"#bbbbbb\", 27, \"#cccccc\", 3, \"#b98eb2\", 8, \"#cccccc\", 36, \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 6 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 8, \"#e0c0c0\", 1, \"#cccccc\", 1 ],\r\n        [ \"#d36971\", 7, \"#cccccc\", 2, \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 2, \"#e0c0c0\", 1, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 2 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 7, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 22, \"#cccccc\", 12, \"#91968b\", 25 ],\r\n        [ \"#d38569\", 22, \"#cccccc\", 12, \"#91968b\", 31 ],\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 4, \"#d38569\", 17, \"#cccccc\", 4, \"#91968b\", 35 ],\r\n        [ \"#d38569\", 23, \"#cccccc\", 11, \"#91968b\", 23 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 8, \"#91968b\", 19 ],\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 8, \"#91968b\", 19 ],\r\n        [ \"#cccccc\", 17, \"#91968b\", 18 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 3, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 7, \"#91968b\", 19 ],\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 4, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 6, \"#91968b\", 19 ],\r\n        [ \"#cccccc\", 13, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 6, \"#91968b\", 18 ],\r\n        [ \"#cccccc\", 1 ]\r\n    ]\r\n}",
	"content/code/30_historic.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\"sl\", [11,12],21],\r\n            [\"sl\", [13,14]],\r\n            [\"sl\", 15],\r\n            [\"sn\", [15,4,\"What does this mean?\"]],\r\n            [\"sm\", [15,4,9,\"negative\"], \"sn\", [15,4,\"error C2106: '=': left operand must be l-value\",\"error\"]],\r\n            [\"sm\", [14,4,5], [15,8,9], \"sn\", [15,4,\"x   is an lvalue\",\"info\"]],\r\n            [\"sm\", [14,8,9], [15,4,5], \"an\", [15,4,\"5/7 is an rvalue\",\"info\"]],\r\n            [\"cn\", \"cm\", \"sl\", [1,5]],\r\n            [\"sl\", [16,17]],\r\n            [\"sm\", [17,22,23]],\r\n            [\"am\", [1,8,13]],\r\n            [\"sn\", [17,4,\"Type of '7' === type of 'a' === int, the category is different.\",\"info\"]],\r\n            [\"cn\", \"cm\", \"sl\", 18],\r\n            [\"sm\", [18,4,9,\"negative\"], \"sn\", [18,4,\"error C3892: 'y': you cannot assign to a variable that is const\",\"error\"]],\r\n            [\"cn\", \"cm\", \"sl\", [6,10]],\r\n            [\"sl\", [19,20]],\r\n            [\"sm\", [20,4,13,\"negative\"], \"sn\", [20,4,\"error C2659: '=': function as left operand\", \"error\"]],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"int sum(int a, int b)\",\r\n        \"{\",\r\n        \"    a = a + b;\",\r\n        \"    return a;\",\r\n        \"}\",\r\n        \"int mul(int a, int b)\",\r\n        \"{\",\r\n        \"    return a * b;\",\r\n        \"}\",\r\n        \"\",\r\n        \"static void example()\",\r\n        \"{\",\r\n        \"    int x;\",\r\n        \"    x = 5;\",\r\n        \"    7 = x;\",\r\n        \"\",\r\n        \"    int const y = sum(7, x);\",\r\n        \"    y = 5;\",\r\n        \"\",\r\n        \"    sum = mul;\",\r\n        \"}\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 4, \"#d38569\", 3, \"#cccccc\", 1, \"#b98eb2\", 4, \"#cccccc\", 3, \"#b98eb2\", 4, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 14 ],\r\n        [ \"#b98eb2\", 11, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 4, \"#d38569\", 3, \"#cccccc\", 1, \"#b98eb2\", 4, \"#cccccc\", 3, \"#b98eb2\", 4, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 11, \"#cccccc\", 6 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 7, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 8, \"#819dc2\", 1, \"#cccccc\", 1 ],\r\n        [ \"#819dc2\", 6, \"#cccccc\", 4 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 14, \"#cccccc\", 4, \"#d38569\", 3, \"#cccccc\", 1, \"#819dc2\", 1, \"#cccccc\", 5 ],\r\n        [ \"#cccccc\", 8, \"#819dc2\", 1, \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#d38569\", 8, \"#cccccc\", 2, \"#d38569\", 3, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 1 ]\r\n    ]\r\n}",
	"content/code/31_rvalue_references.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\"sl\", [1,3], \"sm\", [8,27,55,\"hide\"], [9,27,55,\"hide\"], [10,27,35,\"hide\"], [12,27,65,\"hide\"], [13,27,35,\"hide\"], [14,27,65,\"hide\"], [16,38,66,\"hide\"]],\r\n            [\"sl\", [4,7], 17],\r\n            [\"sl\", 8, \"sn\", [8,4,\"Does what?\"]],\r\n            [\"sn\", [8,4,\"Copies number, still number == 3\",\"info\"]],\r\n            [\"sl\", 9, \"sn\", [9,4,\"Does what?\"], \"cm\", [\"hide\"], \"sm\", [9,27,55,\"hide\"], [10,27,35,\"hide\"], [12,27,65,\"hide\"], [13,27,35,\"hide\"], [14,27,65,\"hide\"], [16,38,66,\"hide\"]],\r\n            [\"sn\", [9,4,\"References number, now number == 4\",\"info\"]],\r\n            [\"sl\", 10, \"sn\", [10,4,\"Does what?\"], \"cm\", [\"hide\"], \"sm\", [10,27,35,\"hide\"], [12,27,65,\"hide\"], [13,27,35,\"hide\"], [14,27,65,\"hide\"], [16,38,66,\"hide\"]],\r\n            [\"sn\", [10,4,\"error: cannot bind rvalue reference of type 'int&&' to lvalue of type 'int'\",\"error\"]],\r\n            [\"sl\", [11,12], \"cn\", \"cm\", [\"hide\"], \"sm\", [12,27,65,\"hide\"], [13,27,35,\"hide\"], [14,27,65,\"hide\"], [16,38,66,\"hide\"]],\r\n            [\"sn\", [12,4,\"Does what?\"]],\r\n            [\"sn\", [12,4,\"Copies temporary object, temporary object == 7\",\"info\"]],\r\n            [\"sl\", 13, \"sn\", [13,4,\"Does what?\"], \"cm\", [\"hide\"], \"sm\", [13,27,35,\"hide\"], [14,27,65,\"hide\"], [16,38,66,\"hide\"]],\r\n            [\"sn\", [13,4,\"error: cannot bind non-const lvalue reference of type 'int&' to an rvalue of type 'int'\",\"error\"]],\r\n            [\"sl\", 14, \"sn\", [14,4,\"Does what?\"], \"cm\", [\"hide\"], \"sm\", [14,27,65,\"hide\"], [16,38,66,\"hide\"]],\r\n            [\"sn\", [14,4,\"References temporary object, temporary object == 8\",\"info\"]],\r\n            [\"cn\", \"cm\", [\"hide\"], \"sm\", [16,38,66,\"hide\"]],\r\n            [\"sl\", [15,16]],\r\n            [\"sm\", [10,4,17], [10,18,24], [16,4,17], [16,28,34]],\r\n            [\"cm\", \"sn\", [16,4,\"References number, number == 5\",\"info\"]],\r\n            [\"cm\", [\"hide\"], \"sn\", [16,4,\"What does std::move actually do?\"]],\r\n            [\"an\", [16,4,\"Hint: There are 2 copies and 0 moves in this function.\",\"info\"]],\r\n            [\"sn\", [16,4,\"std::move turns any expression into an rvalue\",\"info\"]],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"void by_value     (int   x) { ++x; }\",\r\n        \"void by_lvalue_ref(int&  x) { ++x; }\",\r\n        \"void by_rvalue_ref(int&& x) { ++x; }\",\r\n        \"\",\r\n        \"static void example()\",\r\n        \"{\",\r\n        \"    int number = 3;\",\r\n        \"    by_value     (number); // copies    ==> number == 3\",\r\n        \"    by_lvalue_ref(number); // reference ==> number == 4\",\r\n        \"    by_rvalue_ref(number); // error\",\r\n        \"\",\r\n        \"    by_value     (int{7}); // copies    ==> temporary object == 7\",\r\n        \"    by_lvalue_ref(int{7}); // error\",\r\n        \"    by_rvalue_ref(int{7}); // reference ==> temporary object == 8\",\r\n        \"\",\r\n        \"    by_rvalue_ref(std::move(number)); // reference ==> number == 5\",\r\n        \"}\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 5, \"#d38569\", 13, \"#cccccc\", 1, \"#b98eb2\", 6, \"#cccccc\", 11 ],\r\n        [ \"#b98eb2\", 5, \"#d38569\", 13, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 14 ],\r\n        [ \"#b98eb2\", 5, \"#d38569\", 13, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 14 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 7, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 9, \"#819dc2\", 1, \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 17, \"#cccccc\", 10, \"#91968b\", 28 ],\r\n        [ \"#d38569\", 17, \"#cccccc\", 10, \"#91968b\", 28 ],\r\n        [ \"#d38569\", 17, \"#cccccc\", 10, \"#91968b\", 8 ],\r\n        [  ],\r\n        [ \"#d38569\", 17, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 1, \"#819dc2\", 1, \"#cccccc\", 4, \"#91968b\", 38 ],\r\n        [ \"#d38569\", 17, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 1, \"#819dc2\", 1, \"#cccccc\", 4, \"#91968b\", 8 ],\r\n        [ \"#d38569\", 17, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 1, \"#819dc2\", 1, \"#cccccc\", 4, \"#91968b\", 38 ],\r\n        [  ],\r\n        [ \"#d38569\", 17, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 11, \"#91968b\", 28 ],\r\n        [ \"#cccccc\", 1 ]\r\n    ]\r\n}",
	"content/code/32_function_overloading.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\r\n                \"sl\", [1,2], \"sm\",\r\n                [10,13,17,\"hide\"], [11,13,17,\"hide\"], [12,13,17,\"hide\"], [13,13,17,\"hide\"],\r\n                [15,24,28,\"hide\"], [16,24,28,\"hide\"], [17,24,28,\"hide\"], [18,24,28,\"hide\"],\r\n                [20,43,83,\"hide\"], [21,43,69,\"hide\"], [22,43,70,\"hide\"], [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", [3,8], 29],\r\n            [\"sl\", [9,10]],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [11,13,17,\"hide\"], [12,13,17,\"hide\"], [13,13,17,\"hide\"],\r\n                [15,24,28,\"hide\"], [16,24,28,\"hide\"], [17,24,28,\"hide\"], [18,24,28,\"hide\"],\r\n                [20,43,83,\"hide\"], [21,43,69,\"hide\"], [22,43,70,\"hide\"], [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 11],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [12,13,17,\"hide\"], [13,13,17,\"hide\"],\r\n                [15,24,28,\"hide\"], [16,24,28,\"hide\"], [17,24,28,\"hide\"], [18,24,28,\"hide\"],\r\n                [20,43,83,\"hide\"], [21,43,69,\"hide\"], [22,43,70,\"hide\"], [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 12],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [13,13,17,\"hide\"],\r\n                [15,24,28,\"hide\"], [16,24,28,\"hide\"], [17,24,28,\"hide\"], [18,24,28,\"hide\"],\r\n                [20,43,83,\"hide\"], [21,43,69,\"hide\"], [22,43,70,\"hide\"], [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 13],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [15,24,28,\"hide\"], [16,24,28,\"hide\"], [17,24,28,\"hide\"], [18,24,28,\"hide\"],\r\n                [20,43,83,\"hide\"], [21,43,69,\"hide\"], [22,43,70,\"hide\"], [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", [14,15]],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [16,24,28,\"hide\"], [17,24,28,\"hide\"], [18,24,28,\"hide\"],\r\n                [20,43,83,\"hide\"], [21,43,69,\"hide\"], [22,43,70,\"hide\"], [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 16],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [17,24,28,\"hide\"], [18,24,28,\"hide\"],\r\n                [20,43,83,\"hide\"], [21,43,69,\"hide\"], [22,43,70,\"hide\"], [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 17],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [18,24,28,\"hide\"],\r\n                [20,43,83,\"hide\"], [21,43,69,\"hide\"], [22,43,70,\"hide\"], [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 18],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [20,43,83,\"hide\"], [21,43,69,\"hide\"], [22,43,70,\"hide\"], [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", [19,20]],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [21,43,69,\"hide\"], [22,43,70,\"hide\"], [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 21],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [22,43,70,\"hide\"], [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 22],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [23,43,68,\"hide\"],\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 23],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [25,43,95,\"hide\"], [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", [24,25]],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [26,43,94,\"hide\"], [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 26],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [27,43,95,\"hide\"], [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 27],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [28,43,95,\"hide\"],\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            [\"sl\", 28],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [14,4,13,\"hide\"], [19,4,13,\"hide\"], [24,4,13,\"hide\"]\r\n            ],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"inline void fn(T& ) { puts(\\\"l\\\"); }\",\r\n        \"inline void fn(T&&) { puts(\\\"r\\\"); }\",\r\n        \"\",\r\n        \"static void example()\",\r\n        \"{\",\r\n        \"    T   val{};\",\r\n        \"    T&  lvr = val;\",\r\n        \"    T&& rvr = T{};\",\r\n        \"\",\r\n        \"    fn(val); // l\",\r\n        \"    fn(lvr); // l\",\r\n        \"    fn(rvr); // l\",\r\n        \"    fn(T{}); // r\",\r\n        \"    puts(\\\"\\\");\",\r\n        \"    fn(std::move(val)); // r\",\r\n        \"    fn(std::move(lvr)); // r\",\r\n        \"    fn(std::move(rvr)); // r\",\r\n        \"    fn(std::move(T{})); // r\",\r\n        \"    puts(\\\"\\\");\",\r\n        \"    fn(static_cast<decltype(val)>(val));   // r, decltype(val) === T    Copies val!\",\r\n        \"    fn(static_cast<decltype(lvr)>(lvr));   // l, decltype(lvr) === T&\",\r\n        \"    fn(static_cast<decltype(rvr)>(rvr));   // r, decltype(rvr) === T&&\",\r\n        \"    fn(static_cast<decltype(T{})>(T{}));   // r, decltype(T{}) === T\",\r\n        \"    puts(\\\"\\\");\",\r\n        \"    fn(static_cast<decltype(val)&&>(val)); // r, decltype(val) === T    decltype(val)&& === T&&\",\r\n        \"    fn(static_cast<decltype(lvr)&&>(lvr)); // l, decltype(lvr) === T&   decltype(lvr)&& === T&\",\r\n        \"    fn(static_cast<decltype(rvr)&&>(rvr)); // r, decltype(rvr) === T&&  decltype(rvr)&& === T&&\",\r\n        \"    fn(static_cast<decltype(T{})&&>(T{})); // r, decltype(T{}) === T    decltype(T{})&& === T&&\",\r\n        \"}\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 12, \"#d38569\", 2, \"#cccccc\", 1, \"#d3a969\", 1, \"#cccccc\", 6, \"#d38569\", 4, \"#cccccc\", 1, \"#7da76f\", 3, \"#cccccc\", 4 ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 2, \"#cccccc\", 1, \"#d3a969\", 1, \"#cccccc\", 6, \"#d38569\", 4, \"#cccccc\", 1, \"#7da76f\", 3, \"#cccccc\", 4 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 7, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#d3a969\", 8, \"#cccccc\", 6 ],\r\n        [ \"#d3a969\", 5, \"#cccccc\", 13 ],\r\n        [ \"#d3a969\", 5, \"#cccccc\", 9, \"#d3a969\", 1, \"#cccccc\", 3 ],\r\n        [  ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 7, \"#91968b\", 4 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 7, \"#91968b\", 4 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 7, \"#91968b\", 4 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#d3a969\", 1, \"#cccccc\", 5, \"#91968b\", 4 ],\r\n        [ \"#d38569\", 8, \"#cccccc\", 1, \"#7da76f\", 2, \"#cccccc\", 2 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 8, \"#91968b\", 4 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 8, \"#91968b\", 4 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 8, \"#91968b\", 4 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 1, \"#d3a969\", 1, \"#cccccc\", 6, \"#91968b\", 4 ],\r\n        [ \"#d38569\", 8, \"#cccccc\", 1, \"#7da76f\", 2, \"#cccccc\", 2 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 11, \"#cccccc\", 1, \"#b98eb2\", 8, \"#cccccc\", 16, \"#91968b\", 40 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 11, \"#cccccc\", 1, \"#b98eb2\", 8, \"#cccccc\", 16, \"#91968b\", 26 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 11, \"#cccccc\", 1, \"#b98eb2\", 8, \"#cccccc\", 16, \"#91968b\", 27 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 11, \"#cccccc\", 1, \"#b98eb2\", 8, \"#cccccc\", 1, \"#d3a969\", 1, \"#cccccc\", 5, \"#d3a969\", 1, \"#cccccc\", 8, \"#91968b\", 25 ],\r\n        [ \"#d38569\", 8, \"#cccccc\", 1, \"#7da76f\", 2, \"#cccccc\", 2 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 11, \"#cccccc\", 1, \"#b98eb2\", 8, \"#cccccc\", 16, \"#91968b\", 52 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 11, \"#cccccc\", 1, \"#b98eb2\", 8, \"#cccccc\", 16, \"#91968b\", 51 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 11, \"#cccccc\", 1, \"#b98eb2\", 8, \"#cccccc\", 16, \"#91968b\", 52 ],\r\n        [ \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 11, \"#cccccc\", 1, \"#b98eb2\", 8, \"#cccccc\", 1, \"#d3a969\", 1, \"#cccccc\", 7, \"#d3a969\", 1, \"#cccccc\", 6, \"#91968b\", 52 ],\r\n        [ \"#cccccc\", 1 ]\r\n    ]\r\n}",
	"content/code/33_reference_collapsing.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\"sl\", [1,2]],\r\n            [\"sl\", 3],\r\n            [\"sl\", 4],\r\n            [\"sl\", 5],\r\n            [\"sl\", 6],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"using L = int&;\",\r\n        \"using R = int&&;\",\r\n        \"using LL = L&;  // int &  &  ==> int &\",\r\n        \"using LR = L&&; // int &  && ==> int &\",\r\n        \"using RL = R&;  // int && &  ==> int &\",\r\n        \"using RR = R&&; // int && && ==> int &&\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 6, \"#d38569\", 2, \"#cccccc\", 2, \"#b98eb2\", 3, \"#cccccc\", 2 ],\r\n        [ \"#b98eb2\", 6, \"#d38569\", 2, \"#cccccc\", 2, \"#b98eb2\", 3, \"#cccccc\", 3 ],\r\n        [ \"#b98eb2\", 6, \"#d38569\", 3, \"#cccccc\", 2, \"#d38569\", 1, \"#cccccc\", 4, \"#91968b\", 22 ],\r\n        [ \"#b98eb2\", 6, \"#d38569\", 3, \"#cccccc\", 2, \"#d38569\", 1, \"#cccccc\", 4, \"#91968b\", 22 ],\r\n        [ \"#b98eb2\", 6, \"#d38569\", 3, \"#cccccc\", 2, \"#d38569\", 1, \"#cccccc\", 4, \"#91968b\", 22 ],\r\n        [ \"#b98eb2\", 6, \"#d38569\", 3, \"#cccccc\", 2, \"#d38569\", 1, \"#cccccc\", 4, \"#91968b\", 23 ]\r\n    ]\r\n}",
	"content/code/34_xvalue_1.json": "{\r\n    \"lines\": [\r\n        \"static_cast<int&&>(5)\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 11, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 4, \"#819dc2\", 1, \"#cccccc\", 1 ]\r\n    ]\r\n}",
	"content/code/34_xvalue_2.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\"sl\", [1,2]],\r\n            [\"sm\", [2,0,6,\"positive\"]],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"int&& test();\",\r\n        \"test()\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 3, \"#cccccc\", 3, \"#d38569\", 4, \"#cccccc\", 3 ],\r\n        [ \"#d38569\", 4, \"#cccccc\", 2 ]\r\n    ]\r\n}",
	"content/code/34_xvalue_3.json": "{\r\n    \"lines\": [\r\n        \"int&& example() { return 5; } // BAD! Dangling reference\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 3, \"#cccccc\", 3, \"#d38569\", 7, \"#cccccc\", 5, \"#b98eb2\", 7, \"#819dc2\", 1, \"#cccccc\", 4, \"#91968b\", 26 ]\r\n    ]\r\n}",
	"content/code/35_decltype.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\r\n                \"sl\", [4,6], \"sm\",\r\n                [ 8,32,38,\"hide\"], [ 9,32,39,\"hide\"], [11,32,38,\"hide\"], [12,32,38,\"hide\"],\r\n                [14,32,39,\"hide\"], [15,32,39,\"hide\"], [17,32,40,\"hide\"], [18,32,39,\"hide\"],\r\n                [20,32,38,\"hide\"], [21,32,40,\"hide\"], [23,32,62,\"hide\"], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"sl\", [7,8]],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [ 8,35,38       ], [ 9,32,39,\"hide\"], [11,32,38,\"hide\"], [12,32,38,\"hide\"],\r\n                [14,32,39,\"hide\"], [15,32,39,\"hide\"], [17,32,40,\"hide\"], [18,32,39,\"hide\"],\r\n                [20,32,38,\"hide\"], [21,32,40,\"hide\"], [23,32,62,\"hide\"], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"cm\", \"sl\", 9],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [ 9,35,39       ], [11,32,38,\"hide\"], [12,32,38,\"hide\"],\r\n                [14,32,39,\"hide\"], [15,32,39,\"hide\"], [17,32,40,\"hide\"], [18,32,39,\"hide\"],\r\n                [20,32,38,\"hide\"], [21,32,40,\"hide\"], [23,32,62,\"hide\"], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"cm\", \"sl\", [10,11]],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [11,35,38       ], [12,32,38,\"hide\"],\r\n                [14,32,39,\"hide\"], [15,32,39,\"hide\"], [17,32,40,\"hide\"], [18,32,39,\"hide\"],\r\n                [20,32,38,\"hide\"], [21,32,40,\"hide\"], [23,32,62,\"hide\"], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"cm\", \"sl\", 12],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [12,35,38       ],\r\n                [14,32,39,\"hide\"], [15,32,39,\"hide\"], [17,32,40,\"hide\"], [18,32,39,\"hide\"],\r\n                [20,32,38,\"hide\"], [21,32,40,\"hide\"], [23,32,62,\"hide\"], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"cm\", \"sl\", [13,14]],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [14,35,39       ], [15,32,39,\"hide\"], [17,32,40,\"hide\"], [18,32,39,\"hide\"],\r\n                [20,32,38,\"hide\"], [21,32,40,\"hide\"], [23,32,62,\"hide\"], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"cm\", \"sl\", 15],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [15,35,39       ], [17,32,40,\"hide\"], [18,32,39,\"hide\"],\r\n                [20,32,38,\"hide\"], [21,32,40,\"hide\"], [23,32,62,\"hide\"], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"cm\", \"sl\", [16,17]],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [17,35,40       ], [18,32,39,\"hide\"],\r\n                [20,32,38,\"hide\"], [21,32,40,\"hide\"], [23,32,62,\"hide\"], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"cm\", \"sl\", 18],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [18,35,39       ],\r\n                [20,32,38,\"hide\"], [21,32,40,\"hide\"], [23,32,62,\"hide\"], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"cm\"],\r\n            [\"sl\", 1],\r\n            [\"sl\", [19,20]],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [20,35,38       ], [21,32,40,\"hide\"], [23,32,62,\"hide\"], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"cm\", \"sl\", 21],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [21,35,40       ], [23,32,62,\"hide\"], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"cm\", \"sl\", [2,3]],\r\n            [\"sl\", [22,23]],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [23,35,62       ], [24,32,62,\"hide\"]\r\n            ],\r\n            [\"cm\", \"sl\", 24],\r\n            [\r\n                \"cm\", [\"hide\"], \"sm\",\r\n                [24,32,62       ]\r\n            ],\r\n            [\"cm\"],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"struct IntWrapper { int value; };\",\r\n        \"float sum(int const a, float const& b) { return a + b; }\",\r\n        \"\",\r\n        \"int   x = 5;\",\r\n        \"int&  y = x;\",\r\n        \"int&& z = 5;\",\r\n        \"\",\r\n        \"decltype( x )                   // int\",\r\n        \"decltype((x))                   // int&\",\r\n        \"\",\r\n        \"decltype( 5 )                   // int\",\r\n        \"decltype((5))                   // int\",\r\n        \"\",\r\n        \"decltype( y )                   // int&\",\r\n        \"decltype((y))                   // int&\",\r\n        \"\",\r\n        \"decltype( z )                   // int&&\",\r\n        \"decltype((z))                   // int&\",\r\n        \"\",\r\n        \"decltype( IntWrapper{5}.value ) // int\",\r\n        \"decltype((IntWrapper{5}.value)) // int&&\",\r\n        \"\",\r\n        \"decltype( sum )                 // float   (int, float const&)\",\r\n        \"decltype((sum))                 // float(&)(int, float const&)\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 7, \"#d38569\", 11, \"#cccccc\", 2, \"#b98eb2\", 4, \"plum\", 5, \"#cccccc\", 4 ],\r\n        [ \"#b98eb2\", 6, \"#d38569\", 3, \"#cccccc\", 1, \"#b98eb2\", 10, \"#cccccc\", 3, \"#b98eb2\", 11, \"#cccccc\", 7, \"#b98eb2\", 7, \"#cccccc\", 8 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 6, \"#cccccc\", 4, \"#819dc2\", 1, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 3, \"#cccccc\", 9 ],\r\n        [ \"#b98eb2\", 3, \"#cccccc\", 7, \"#819dc2\", 1, \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 24, \"#91968b\", 6 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 24, \"#91968b\", 7 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 2, \"#819dc2\", 2, \"#cccccc\", 20, \"#91968b\", 6 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 2, \"#819dc2\", 1, \"#cccccc\", 21, \"#91968b\", 6 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 24, \"#91968b\", 7 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 24, \"#91968b\", 7 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 24, \"#91968b\", 8 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 24, \"#91968b\", 7 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 2, \"#d38569\", 10, \"#cccccc\", 1, \"#819dc2\", 1, \"#cccccc\", 2, \"plum\", 6, \"#cccccc\", 2, \"#91968b\", 6 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 2, \"#d38569\", 10, \"#cccccc\", 1, \"#819dc2\", 1, \"#cccccc\", 2, \"plum\", 5, \"#cccccc\", 3, \"#91968b\", 8 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 18, \"#91968b\", 30 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 2, \"#d38569\", 3, \"#cccccc\", 19, \"#91968b\", 30 ]\r\n    ]\r\n}",
	"content/code/40_creating_components.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\"sl\", [1,6], \"sm\", [54,50,66,\"hide\"], [54,71,72,\"hide\"]],\r\n            [\"sl\", [13,14], [23,25]],\r\n            [\"sl\", [7,12]],\r\n            [\"sl\", [15,22]],\r\n            [\"hl\", [1,25]],\r\n            [\"sl\", [27,28], [66,68]],\r\n            [\"sl\", [86,88], 91],\r\n            [\"sl\", 89],\r\n            [\"sm\", [89,28,67]],\r\n            [\"sm\", [89,50,64]],\r\n            [\"cm\", \"sl\", 90, \"sn\", [90,4,\"Fills the particle system with data.\", \"info\"]],\r\n            [\"sn\", [89,28,\"How to write create_component?\"]],\r\n            [\"sl\", [29,30]],\r\n            [\"sl\", [31,32], 35],\r\n            [\"sl\", 33],\r\n            [\"sl\", 34],\r\n            [\"cn\"],\r\n            [\"hl\", [86,91], \"sl\", [93,96], 103],\r\n            [\"sl\", [97,99]],\r\n            [\"sm\", [99,26,40]],\r\n            [\"sm\", [99,42,44]],\r\n            [\"cm\", \"hl\", [30,35], \"sl\", [37,42]],\r\n            [\"sm\", [37,26,36]],\r\n            [\"sm\", [38,24,31]],\r\n            [\"sm\", [40,50,66]],\r\n            [\"cm\", \"sn\", [99,4,\"What happens with ps? (Copies, moves, etc.)\"]],\r\n            [\"sm\", [38,24,31,\"positive\"], [99,42,44]],\r\n            [\"sm\", [38,24,31], [40,50,66,\"positive\"]],\r\n            [\"cm\", \"sn\", [99,4,\"1st copy, 2nd move (or copy if non-moveable).\",\"info\"]],\r\n            [\"an\", [99,4,\"Ideally always 1 copy.\",\"info\"]],\r\n            [\"cn\", \"hl\", [37,42], \"sl\", [44,49]],\r\n            [\"sm\", [45,26,32]],\r\n            [\"sm\", [47,50,55]],\r\n            [\"cm\", \"sn\", [99,4,\"What happens?\"]],\r\n            [\"sm\", [45,24,38,\"positive\"], [99,42,44]],\r\n            [\"sm\", [45,24,38], [47,50,55,\"positive\"]],\r\n            [\"cm\", \"sn\", [99,4,\"1st reference (no-op), 2nd copy.\",\"info\"]],\r\n            [\"cn\", \"sl\", [100,102]],\r\n            [\"sm\", [102,50,63]],\r\n            [\"cm\", \"sn\", [102,4,\"What happens?\"]],\r\n            [\"sm\", [45,24,38,\"positive\"], [102,50,63]],\r\n            [\"sm\", [45,24,38], [47,50,55,\"positive\"]],\r\n            [\"cm\", \"sn\", [102,4,\"1st reference (no-op), 2nd copy.\", \"info\"]],\r\n            [\"sn\", [47,8,\"What if we change it to std::move(param)?\"]],\r\n            [\r\n                \"sn\", [47,8,\"Would be xvalue of type: P const&&\",\"info\"],\r\n                [47,8,\"Cannot call: T(P&&). Can call: T(P const&). Copies value.\",\"info\"]\r\n            ],\r\n            [\"cn\", \"hl\", [44,49], \"sl\", [51,53], 56],\r\n            [\"sm\", [52,24,27]],\r\n            [\"am\", [99,42,44,\"negative\"], \"sn\", [99,4,\"Does this compile?\"]],\r\n            [\"sm\", [52,24,27], [99,42,44,\"positive\"], \"sn\", [99,4,\"Yes! P === ParticleSystem& ==> P&& === ParticleSystem&\",\"info\"]],\r\n            [\"an\", [52,12,\"ParticleSystem& param\",\"info\"]],\r\n            [\"an\", [52,12,\"ONLY with rvalue reference to deduced type, special name \\\"forwarding reference\\\".\",\"warning\"]],\r\n            [\"cm\", \"cn\", \"sl\", [54,55]],\r\n            [\"sn\", [99,4,\"What happens?\"]],\r\n            [\"sn\", [99,4,\"1 copy, identical to calling ParticleSystem{ps}\",\"info\"]],\r\n            [\"sn\", [102,4,\"What happens?\"]],\r\n            [\"an\", [52,15,\"Category of 'param'?\"]],\r\n            [\"sn\", [52,8,\"Category: lvalue, type: ParticleSystem&&.\",\"info\"]],\r\n            [\"sn\", [54,8,\"What should wrap 'param'?\"]],\r\n            [\"an\", [54,8,\"Hint: param's type is either ParticleSystem& or ParticleSystem&&\"]],\r\n            [\"an\", [54,8,\"static_cast<decltype(param)>(param) would work\",\"info\"]],\r\n            [\"cm\", [\"hide\"], \"sm\", [54,50,66], [54,71,72], \"sn\", [54,8,\"Or use std::forward<P>.\",\"info\"]],\r\n            [\"cm\", \"cn\"],\r\n            [\"hl\", [51,56], [93,103], \"sl\", [58,60], [62,65]],\r\n            [\"sm\", [58,34,37], [59,27,30], [62,73,76]],\r\n            [\"cm\", \"sl\", [105,107], 109],\r\n            [\"sl\", 108],\r\n            [\"sn\", [108,4,\"example.cpp(62): error C2664: 'void std::vector<std::unique_ptr<Component,std::default_delete<_Ty>>,std::allocator<std::unique_ptr<_Ty,std::default_delete<_Ty>>>>::push_back(std::unique_ptr<_Ty,std::default_delete<_Ty>> &&)': cannot convert argument 1 from 'std::unique_ptr<T,std::default_delete<_Ty>>' to 'const std::unique_ptr<Component,std::default_delete<_Ty>> &'\",\"error\"]],\r\n            [\"cn\", \"sl\", 61, \"sm\", [61,8,80]],\r\n            [\"sm\", [61,22,53]],\r\n            [\"cm\", \"sn\", [108,4,\"example.cpp(61): error C2338: T must be a Component [with T=int]\",\"error\"]],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"class Component\",\r\n        \"{\",\r\n        \"public:\",\r\n        \"    virtual void update(float delta_seconds) = 0;\",\r\n        \"    virtual ~Component() noexcept = default;\",\r\n        \"};\",\r\n        \"\",\r\n        \"struct Particle\",\r\n        \"{\",\r\n        \"    float x, y;\",\r\n        \"    float vx, vy;\",\r\n        \"};\",\r\n        \"class ParticleSystem : public Component\",\r\n        \"{\",\r\n        \"    void update(float const delta_seconds) override\",\r\n        \"    {\",\r\n        \"        for (auto& particle : particles)\",\r\n        \"        {\",\r\n        \"            particle.x += particle.vx * delta_seconds;\",\r\n        \"            particle.y += particle.vy * delta_seconds;\",\r\n        \"        }\",\r\n        \"    }\",\r\n        \"public:\",\r\n        \"    std::vector<Particle> particles;\",\r\n        \"};\",\r\n        \"\",\r\n        \"class GameObject\",\r\n        \"{\",\r\n        \"public:\",\r\n        \"    template <typename T>\",\r\n        \"    T& create_component()\",\r\n        \"    {\",\r\n        \"        components_.push_back(std::make_unique<T>());\",\r\n        \"        return static_cast<T&>(*components_.back());\",\r\n        \"    }\",\r\n        \"\",\r\n        \"    template <typename T, typename P>\",\r\n        \"    T& create_component(P param)\",\r\n        \"    {\",\r\n        \"        components_.push_back(std::make_unique<T>(std::move(param)));\",\r\n        \"        return static_cast<T&>(*components_.back());\",\r\n        \"    }\",\r\n        \"\",\r\n        \"    template <typename T, typename P>\",\r\n        \"    T& create_component(P const& param)\",\r\n        \"    {\",\r\n        \"        components_.push_back(std::make_unique<T>(param));\",\r\n        \"        return static_cast<T&>(*components_.back());\",\r\n        \"    }\",\r\n        \"\",\r\n        \"    template <typename T, typename P>\",\r\n        \"    T& create_component(P&& param)\",\r\n        \"    {\",\r\n        \"        components_.push_back(std::make_unique<T>(std::forward<P>(param)));\",\r\n        \"        return static_cast<T&>(*components_.back());\",\r\n        \"    }\",\r\n        \"\",\r\n        \"    template <typename T, typename... P>\",\r\n        \"    T& create_component(P&&... params)\",\r\n        \"    {\",\r\n        \"        static_assert(std::is_base_of_v<Component, T>, \\\"T must be a Component\\\");\",\r\n        \"        components_.push_back(std::make_unique<T>(std::forward<P>(params)...));\",\r\n        \"        return static_cast<T&>(*components_.back());\",\r\n        \"    }\",\r\n        \"\",\r\n        \"private:\",\r\n        \"    std::vector<std::unique_ptr<Component>> components_;\",\r\n        \"};\",\r\n        \"\",\r\n        \"void create_fire_particles(ParticleSystem& ps)\",\r\n        \"{\",\r\n        \"    ps.particles.clear();\",\r\n        \"    std::default_random_engine rnd;\",\r\n        \"    for (int i = 0; i < 100; i++)\",\r\n        \"    {\",\r\n        \"        ps.particles.push_back(\",\r\n        \"        {\",\r\n        \"            /*  x */std::uniform_real_distribution<float>{ -0.2f, 0.2f }(rnd),\",\r\n        \"            /*  y */std::uniform_real_distribution<float>{ -0.2f, 0.2f }(rnd),\",\r\n        \"            /* vx */std::uniform_real_distribution<float>{ -0.2f, 0.2f }(rnd),\",\r\n        \"            /* vy */std::uniform_real_distribution<float>{  2.0f, 8.0f }(rnd),\",\r\n        \"        });\",\r\n        \"    }\",\r\n        \"}\",\r\n        \"\",\r\n        \"void example_basic()\",\r\n        \"{\",\r\n        \"    GameObject fire;\",\r\n        \"    auto& particle_system = fire.create_component<ParticleSystem>();\",\r\n        \"    create_fire_particles(particle_system);\",\r\n        \"}\",\r\n        \"\",\r\n        \"void example_multiple()\",\r\n        \"{\",\r\n        \"    ParticleSystem ps;\",\r\n        \"    create_fire_particles(ps);\",\r\n        \"\",\r\n        \"    GameObject fire;\",\r\n        \"    fire.create_component<ParticleSystem>(ps);\",\r\n        \"\",\r\n        \"    GameObject another_fire;\",\r\n        \"    another_fire.create_component<ParticleSystem>(std::move(ps));\",\r\n        \"}\",\r\n        \"\",\r\n        \"void example_error()\",\r\n        \"{\",\r\n        \"    GameObject fire;\",\r\n        \"    fire.create_component<int>(5);\",\r\n        \"}\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 6, \"#d38569\", 9 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 6, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 17, \"#c8c8c8\", 6, \"#cccccc\", 1, \"#b98eb2\", 6, \"#cccccc\", 17, \"#819dc2\", 1, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 12, \"#c8c8c8\", 10, \"#cccccc\", 3, \"#b98eb2\", 9, \"#cccccc\", 2, \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 2 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 7, \"#d38569\", 8 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 10, \"plum\", 1, \"#cccccc\", 2, \"plum\", 1, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 10, \"plum\", 2, \"#cccccc\", 2, \"plum\", 2, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 2 ],\r\n        [ \"#b98eb2\", 6, \"#d38569\", 15, \"#cccccc\", 2, \"#b98eb2\", 7, \"#d38569\", 9 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 9, \"#c8c8c8\", 6, \"#cccccc\", 1, \"#b98eb2\", 12, \"#cccccc\", 15, \"#b98eb2\", 8 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#b98eb2\", 12, \"#cccccc\", 1, \"#b98eb2\", 4, \"#cccccc\", 13, \"#e0c0c0\", 9, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 9 ],\r\n        [ \"#cccccc\", 21, \"plum\", 2, \"#cccccc\", 12, \"plum\", 3, \"#cccccc\", 16 ],\r\n        [ \"#cccccc\", 21, \"plum\", 2, \"#cccccc\", 12, \"plum\", 3, \"#cccccc\", 16 ],\r\n        [ \"#cccccc\", 9 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#b98eb2\", 6, \"#cccccc\", 1 ],\r\n        [ \"#d36971\", 7, \"#cccccc\", 2, \"#d38569\", 6, \"#cccccc\", 1, \"#d38569\", 8, \"#cccccc\", 2, \"#e0c0c0\", 9, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 2 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 6, \"#d38569\", 10 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 6, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 13, \"#cccccc\", 1, \"#b98eb2\", 9, \"#d3a869\", 1, \"#cccccc\", 1 ],\r\n        [ \"#d3a869\", 5, \"#cccccc\", 2, \"#c8c8c8\", 16, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#e0c0c0\", 19, \"#cccccc\", 1, \"#bbbbbb\", 9, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 5 ],\r\n        [ \"#b98eb2\", 26, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 4, \"#e0c0c0\", 11, \"#cccccc\", 1, \"#c8c8c8\", 4, \"#cccccc\", 4 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 13, \"#cccccc\", 1, \"#b98eb2\", 9, \"#d3a869\", 1, \"#cccccc\", 2, \"#b98eb2\", 9, \"#d3a869\", 1, \"#cccccc\", 1 ],\r\n        [ \"#d3a869\", 5, \"#cccccc\", 2, \"#c8c8c8\", 16, \"#cccccc\", 1, \"#d3a869\", 2, \"#cccccc\", 6 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#e0c0c0\", 19, \"#cccccc\", 1, \"#bbbbbb\", 9, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 10 ],\r\n        [ \"#b98eb2\", 26, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 4, \"#e0c0c0\", 11, \"#cccccc\", 1, \"#c8c8c8\", 4, \"#cccccc\", 4 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 13, \"#cccccc\", 1, \"#b98eb2\", 9, \"#d3a869\", 1, \"#cccccc\", 2, \"#b98eb2\", 9, \"#d3a869\", 1, \"#cccccc\", 1 ],\r\n        [ \"#d3a869\", 5, \"#cccccc\", 2, \"#c8c8c8\", 16, \"#cccccc\", 1, \"#d3a869\", 2, \"#b98eb2\", 5, \"#cccccc\", 8 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#e0c0c0\", 19, \"#cccccc\", 1, \"#bbbbbb\", 9, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 10 ],\r\n        [ \"#b98eb2\", 26, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 4, \"#e0c0c0\", 11, \"#cccccc\", 1, \"#c8c8c8\", 4, \"#cccccc\", 4 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 13, \"#cccccc\", 1, \"#b98eb2\", 9, \"#d3a869\", 1, \"#cccccc\", 2, \"#b98eb2\", 9, \"#d3a869\", 1, \"#cccccc\", 1 ],\r\n        [ \"#d3a869\", 5, \"#cccccc\", 2, \"#c8c8c8\", 16, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 9 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#e0c0c0\", 19, \"#cccccc\", 1, \"#bbbbbb\", 9, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 7, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 11 ],\r\n        [ \"#b98eb2\", 26, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 4, \"#e0c0c0\", 11, \"#cccccc\", 1, \"#c8c8c8\", 4, \"#cccccc\", 4 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 13, \"#cccccc\", 1, \"#b98eb2\", 9, \"#d3a869\", 1, \"#cccccc\", 2, \"#b98eb2\", 8, \"#cccccc\", 4, \"#d3a869\", 1, \"#cccccc\", 1 ],\r\n        [ \"#d3a869\", 5, \"#cccccc\", 2, \"#c8c8c8\", 16, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 13 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#b98eb2\", 21, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 15, \"#d38569\", 9, \"#cccccc\", 2, \"#d3a869\", 1, \"#cccccc\", 3, \"#7da76f\", 23, \"#cccccc\", 2 ],\r\n        [ \"#e0c0c0\", 19, \"#cccccc\", 1, \"#bbbbbb\", 9, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 7, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 15 ],\r\n        [ \"#b98eb2\", 26, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 4, \"#e0c0c0\", 11, \"#cccccc\", 1, \"#c8c8c8\", 4, \"#cccccc\", 4 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 7, \"#cccccc\", 1 ],\r\n        [ \"#d36971\", 7, \"#cccccc\", 2, \"#d38569\", 6, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 10, \"#cccccc\", 1, \"#d38569\", 9, \"#cccccc\", 3, \"#e0c0c0\", 11, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 2 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 5, \"#d38569\", 21, \"#cccccc\", 1, \"#d38569\", 14, \"#cccccc\", 5 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 7, \"#e0c0c0\", 9, \"#cccccc\", 1, \"#c8c8c8\", 5, \"#cccccc\", 3 ],\r\n        [ \"#d36971\", 7, \"#cccccc\", 2, \"#d38569\", 22, \"#cccccc\", 4 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 1, \"#b98eb2\", 4, \"#cccccc\", 4, \"#819dc2\", 1, \"#cccccc\", 6, \"#819dc2\", 3, \"#cccccc\", 6 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#cccccc\", 11, \"#e0c0c0\", 9, \"#cccccc\", 1, \"#c8c8c8\", 9, \"#cccccc\", 1 ],\r\n        [ \"#cccccc\", 9 ],\r\n        [ \"#91968b\", 20, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 25, \"#cccccc\", 1, \"#b98eb2\", 5, \"#cccccc\", 4, \"#819dc2\", 4, \"#cccccc\", 2, \"#819dc2\", 5, \"#cccccc\", 7 ],\r\n        [ \"#91968b\", 20, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 25, \"#cccccc\", 1, \"#b98eb2\", 5, \"#cccccc\", 4, \"#819dc2\", 4, \"#cccccc\", 2, \"#819dc2\", 5, \"#cccccc\", 7 ],\r\n        [ \"#91968b\", 20, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 25, \"#cccccc\", 1, \"#b98eb2\", 5, \"#cccccc\", 4, \"#819dc2\", 4, \"#cccccc\", 2, \"#819dc2\", 5, \"#cccccc\", 7 ],\r\n        [ \"#91968b\", 20, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 25, \"#cccccc\", 1, \"#b98eb2\", 5, \"#cccccc\", 4, \"#819dc2\", 4, \"#cccccc\", 2, \"#819dc2\", 5, \"#cccccc\", 7 ],\r\n        [ \"#cccccc\", 11 ],\r\n        [ \"#cccccc\", 5 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 5, \"#d38569\", 13, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 15, \"#cccccc\", 5 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 25, \"#c8c8c8\", 16, \"#cccccc\", 1, \"#d38569\", 14, \"#cccccc\", 4 ],\r\n        [ \"#d38569\", 25, \"#cccccc\", 18 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 5, \"#d38569\", 16, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 19, \"#cccccc\", 3 ],\r\n        [ \"#d38569\", 25, \"#cccccc\", 5 ],\r\n        [  ],\r\n        [ \"#d38569\", 15, \"#cccccc\", 5 ],\r\n        [ \"#cccccc\", 9, \"#bbbbbb\", 16, \"#cccccc\", 1, \"#d38569\", 14, \"#cccccc\", 6 ],\r\n        [  ],\r\n        [ \"#d38569\", 15, \"#cccccc\", 13 ],\r\n        [ \"#cccccc\", 17, \"#bbbbbb\", 16, \"#cccccc\", 1, \"#d38569\", 14, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 6 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 5, \"#d38569\", 13, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#d38569\", 15, \"#cccccc\", 5 ],\r\n        [ \"#cccccc\", 9, \"#bbbbbb\", 16, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 2, \"#819dc2\", 1, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ]\r\n    ]\r\n}",
	"content/code/41_return_values.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\"sl\", [11,13], \"sm\", [6,0,14,\"hide\"]],\r\n            [\"sl\", [14,17], 28],\r\n            [\"sl\", [18,19]],\r\n            [\"sl\", 20],\r\n            [\"sl\", 21],\r\n            [\"sl\", 22],\r\n            [\"sl\", [23,27]],\r\n            [\"sm\", [19,30,44], [24,30,57]],\r\n            [\"sm\", [20,30,44], [25,30,57]],\r\n            [\"sm\", [21,30,44], [26,30,57]],\r\n            [\"sm\", [22,30,48], [27,30,61]],\r\n            [\"cm\", \"sl\", [5,10], \"sn\", [9,0,\"What return type?\"]],\r\n            [\"cn\", \"cm\", [\"hide\"], \"sm\", [6,0,14]],\r\n            [\"cm\", \"sn\", [27,4,\"How to make IS_LVALUE, IS_XVALUE, IS_PRVALUE?\"]],\r\n            [\"cn\", \"sl\", 1],\r\n            [\"sl\", 2],\r\n            [\"sl\", 3],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"#define  IS_LVALUE(EXPR) ( std::is_lvalue_reference_v<decltype((EXPR))>)\",\r\n        \"#define  IS_XVALUE(EXPR) ( std::is_rvalue_reference_v<decltype((EXPR))>)\",\r\n        \"#define IS_PRVALUE(EXPR) (!std::is_reference_v       <decltype((EXPR))>)\",\r\n        \"\",\r\n        \"template <typename Fun, typename... P>\",\r\n        \"decltype(auto) passthrough(Fun fn, P&&... v)\",\r\n        \"{\",\r\n        \"    return fn(std::forward<P>(v)...);\",\r\n        \"}\",\r\n        \"\",\r\n        \"int   idt_integer(int& v) { return v; }\",\r\n        \"int&  lvr_integer(int& v) { return v; }\",\r\n        \"int&& rvr_integer(int& v) { return static_cast<int&&>(v); }\",\r\n        \"\",\r\n        \"static void example()\",\r\n        \"{\",\r\n        \"    int x{ 5 };\",\r\n        \"\",\r\n        \"    static_assert(IS_PRVALUE( idt_integer(x) ));\",\r\n        \"    static_assert( IS_LVALUE( lvr_integer(x) ));\",\r\n        \"    static_assert( IS_XVALUE( rvr_integer(x) ));\",\r\n        \"    static_assert( IS_XVALUE( std::move<int&>(x) ));\",\r\n        \"\",\r\n        \"    static_assert(IS_PRVALUE( passthrough(idt_integer, x) ));\",\r\n        \"    static_assert( IS_LVALUE( passthrough(lvr_integer, x) ));\",\r\n        \"    static_assert( IS_XVALUE( passthrough(rvr_integer, x) ));\",\r\n        \"    static_assert( IS_XVALUE( passthrough(std::move<int&>, x) ));\",\r\n        \"}\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#9b9b9b\", 9, \"#d3a969\", 9, \"#cccccc\", 1, \"#bbbbbb\", 4, \"#cccccc\", 4, \"#d36971\", 3, \"#cccccc\", 24, \"#b98eb2\", 8, \"#cccccc\", 2, \"#bbbbbb\", 4, \"#cccccc\", 4 ],\r\n        [ \"#9b9b9b\", 9, \"#d3a969\", 9, \"#cccccc\", 1, \"#bbbbbb\", 4, \"#cccccc\", 4, \"#d36971\", 3, \"#cccccc\", 24, \"#b98eb2\", 8, \"#cccccc\", 2, \"#bbbbbb\", 4, \"#cccccc\", 4 ],\r\n        [ \"#9b9b9b\", 8, \"#d3a969\", 10, \"#cccccc\", 1, \"#bbbbbb\", 4, \"#cccccc\", 4, \"#d36971\", 3, \"#cccccc\", 24, \"#b98eb2\", 8, \"#cccccc\", 2, \"#bbbbbb\", 4, \"#cccccc\", 4 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 1, \"#b98eb2\", 9, \"#d3a869\", 3, \"#cccccc\", 2, \"#b98eb2\", 8, \"#cccccc\", 4, \"#d3a869\", 1, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 1, \"#b98eb2\", 4, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d3a869\", 4, \"#cccccc\", 4, \"#d3a869\", 1, \"#cccccc\", 8 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 11, \"#cccccc\", 3, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 7, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 9 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 6, \"#d38569\", 11, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 7, \"#b98eb2\", 7, \"#cccccc\", 4 ],\r\n        [ \"#b98eb2\", 3, \"#cccccc\", 3, \"#d38569\", 11, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 7, \"#b98eb2\", 7, \"#cccccc\", 4 ],\r\n        [ \"#b98eb2\", 3, \"#cccccc\", 3, \"#d38569\", 11, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 7, \"#b98eb2\", 18, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 9 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 12, \"#d38569\", 7, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 8, \"#cccccc\", 3, \"#819dc2\", 2, \"#cccccc\", 2 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 17, \"#cccccc\", 1, \"#d3a969\", 10, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 7 ],\r\n        [ \"#b98eb2\", 17, \"#cccccc\", 2, \"#d3a969\", 9, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 7 ],\r\n        [ \"#b98eb2\", 17, \"#cccccc\", 2, \"#d3a969\", 9, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 7 ],\r\n        [ \"#b98eb2\", 17, \"#cccccc\", 2, \"#d3a969\", 9, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 9 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 17, \"#cccccc\", 1, \"#d3a969\", 10, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d38569\", 11, \"#cccccc\", 8 ],\r\n        [ \"#b98eb2\", 17, \"#cccccc\", 2, \"#d3a969\", 9, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d38569\", 11, \"#cccccc\", 8 ],\r\n        [ \"#b98eb2\", 17, \"#cccccc\", 2, \"#d3a969\", 9, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d38569\", 11, \"#cccccc\", 8 ],\r\n        [ \"#b98eb2\", 17, \"#cccccc\", 2, \"#d3a969\", 9, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 4, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 10 ],\r\n        [ \"#cccccc\", 1 ]\r\n    ]\r\n}",
	"content/code/42_performance.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\"sl\", [1,4]],\r\n            [\"sl\", [5,9]],\r\n            [\"sl\", [10,14]],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"std::vector<int> generate_numbers()\",\r\n        \"{\",\r\n        \"    return { 0, 1, 2, 3, 4, 5 };\",\r\n        \"}\",\r\n        \"\",\r\n        \"std::vector<int> passthrough()\",\r\n        \"{\",\r\n        \"    return generate_numbers();\",\r\n        \"}\",\r\n        \"\",\r\n        \"void example()\",\r\n        \"{\",\r\n        \"    auto numbers = passthrough();\",\r\n        \"}\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 2, \"#d38569\", 16, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 11, \"#cccccc\", 2, \"#819dc2\", 1, \"#cccccc\", 2, \"#819dc2\", 1, \"#cccccc\", 2, \"#819dc2\", 1, \"#cccccc\", 2, \"#819dc2\", 1, \"#cccccc\", 2, \"#819dc2\", 1, \"#cccccc\", 2, \"#819dc2\", 2, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 6, \"#cccccc\", 1, \"#b98eb2\", 3, \"#cccccc\", 2, \"#d38569\", 11, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 11, \"#d38569\", 16, \"#cccccc\", 3 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 5, \"#d38569\", 7, \"#cccccc\", 2 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 10, \"#d38569\", 11, \"#cccccc\", 3 ],\r\n        [ \"#cccccc\", 1 ]\r\n    ]\r\n}",
	"content/code/43_move_forward.json": "{\r\n    \"sets\": {\r\n        \"main\": [\r\n            [\"sl\", [1,2]],\r\n            [\"sm\", [1,10,20], [2,44,47], \"sn\", [2,44,\"Forwarding reference\",\"info\"]],\r\n            [\"sl\", [3,5], \"sm\", [2,10,36], [4,23,49], \"sn\", [4,23,\"Why remove reference from T?\"]],\r\n            [\"sn\", [4,23,\"T& && would otherwise collapse to T&\",\"info\"]],\r\n            [\"cm\", \"cn\", \"sl\", [6,8], [13,14]],\r\n            [\"sn\", [8,0,\"Called if argument is lvalue. (Almost always.)\",\"info\"]],\r\n            [\"an\", [14,0,\"Called if argument is rvalue. (Almost never.)\",\"info\"]],\r\n            [\"sl\", [9,12], \"cn\"],\r\n            [\"sm\", [8,10,13], [10,23,26]],\r\n            [\"sn\", [10,4,\"If T ==> T&&. If T& ==> T&. If T&& ==> T&&.\",\"info\"]],\r\n            [\"cn\", \"cm\", \"sl\", [15,18]],\r\n            [\"sn\", [16,4,\"Value being forwarded is an rvalue. Cannot convert rvalue to lvalue.\",\"info\"]],\r\n            [],\r\n            [\"cn\"],\r\n            null\r\n        ]\r\n    },\r\n    \"lines\": [\r\n        \"template <typename T>\",\r\n        \"constexpr std::remove_reference_t<T>&& move(T&& v) noexcept\",\r\n        \"{\",\r\n        \"    return static_cast<std::remove_reference_t<T>&&>(v);\",\r\n        \"}\",\r\n        \"\",\r\n        \"template <typename T>\",\r\n        \"constexpr T&& forward(std::remove_reference_t<T>& v) noexcept\",\r\n        \"{\",\r\n        \"    return static_cast<T&&>(v);\",\r\n        \"}\",\r\n        \"\",\r\n        \"template <typename T>\",\r\n        \"constexpr T&& forward(std::remove_reference_t<T>&& v) noexcept\",\r\n        \"{\",\r\n        \"    static_assert(!std::is_lvalue_reference_v<T>, \\\"incorrect template parameter to T\\\");\",\r\n        \"    return static_cast<T&&>(v);\",\r\n        \"}\"\r\n    ],\r\n    \"tokens\": [\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 1, \"#b98eb2\", 9, \"#d3a869\", 1, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 10, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 18, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 4, \"#d38569\", 4, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 6, \"#b98eb2\", 8 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 22, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 18, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 8 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 1, \"#b98eb2\", 9, \"#d3a869\", 1, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 10, \"#d3a869\", 1, \"#cccccc\", 3, \"#d38569\", 7, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 18, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 6, \"#b98eb2\", 8 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 22, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 7 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [  ],\r\n        [ \"#b98eb2\", 9, \"#cccccc\", 1, \"#b98eb2\", 9, \"#d3a869\", 1, \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 10, \"#d3a869\", 1, \"#cccccc\", 3, \"#d38569\", 7, \"#cccccc\", 1, \"#d36971\", 3, \"#cccccc\", 2, \"#d38569\", 18, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 7, \"#b98eb2\", 8 ],\r\n        [ \"#cccccc\", 1 ],\r\n        [ \"#b98eb2\", 17, \"#cccccc\", 2, \"#d36971\", 3, \"#cccccc\", 24, \"#d3a869\", 1, \"#cccccc\", 3, \"#7da76f\", 35, \"#cccccc\", 2 ],\r\n        [ \"#b98eb2\", 22, \"#cccccc\", 1, \"#d3a869\", 1, \"#cccccc\", 7 ],\r\n        [ \"#cccccc\", 1 ]\r\n    ]\r\n}"
}